@@ -32,9 +32,11 @@
 #include "gsystem-glib-compat.h"
 #include <glib/gstdio.h>
 #include <gio/gunixinputstream.h>
+#include <gio/gfiledescriptorbased.h>
 #include <gio/gunixoutputstream.h>
 #include <glib-unix.h>
 #include <limits.h>
+#include <dirent.h>
 
 static int
 close_nointr (int fd)
@@ -66,23 +68,6 @@ open_nointr (const char *path, int flags, mode_t mode)
   return res;
 }
 
-static int
-_open_fd_noatime (const char *path)
-{
-  int fd;
-
-#ifdef O_NOATIME
-  fd = open_nointr (path, O_RDONLY | O_NOATIME, 0);
-  /* Only the owner or superuser may use O_NOATIME; so we may get
-   * EPERM.  EINVAL may happen if the kernel is really old...
-   */
-  if (fd == -1 && (errno == EPERM || errno == EINVAL))
-#endif
-    fd = open_nointr (path, O_RDONLY, 0);
-  
-  return fd;
-}
-
 static inline void
 _set_error_from_errno (GError **error)
 {
@@ -91,6 +76,50 @@ _set_error_from_errno (GError **error)
                        g_strerror (errsv));
 }
 
+/**
+ * gs_file_openat_noatime:
+ * @dfd: File descriptor for directory
+ * @name: Pathname, relative to @dfd
+ * @ret_fd: (out): Returned file descriptor
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Wrapper for openat() using %O_RDONLY with %O_NOATIME if available.
+ */
+gboolean
+gs_file_openat_noatime (int            dfd,
+                        const char    *name,
+                        int           *ret_fd,
+                        GCancellable  *cancellable,
+                        GError       **error)
+{
+  int fd;
+
+#ifdef O_NOATIME
+  do
+    fd = openat (dfd, name, O_RDONLY | O_NOATIME, 0);
+  while (G_UNLIKELY (fd == -1 && errno == EINTR));
+  /* Only the owner or superuser may use O_NOATIME; so we may get
+   * EPERM.  EINVAL may happen if the kernel is really old...
+   */
+  if (fd == -1 && (errno == EPERM || errno == EINVAL))
+#endif
+    do
+      fd = openat (dfd, name, O_RDONLY, 0);
+    while (G_UNLIKELY (fd == -1 && errno == EINTR));
+  
+  if (fd == -1)
+    {
+      _set_error_from_errno (error);
+      return FALSE;
+    }
+  else
+    {
+      *ret_fd = fd;
+      return TRUE;
+    }
+}
+
 /**
  * gs_file_read_noatime:
  * @file: a #GFile
@@ -116,18 +145,59 @@ gs_file_read_noatime (GFile         *file,
 
   path = gs_file_get_path_cached (file);
   if (path == NULL)
-    return NULL;
-
-  fd = _open_fd_noatime (path);
-  if (fd < 0)
     {
-      _set_error_from_errno (error);
+      char *uri;
+      uri = g_file_get_uri (file);
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
+                   "%s has no associated path", uri);
+      g_free (uri);
       return NULL;
     }
 
+  if (!gs_file_openat_noatime (AT_FDCWD, path, &fd, cancellable, error))
+    return NULL;
+
   return g_unix_input_stream_new (fd, TRUE);
 }
 
+/**
+ * gs_stream_fstat:
+ * @stream: A stream containing a Unix file descriptor
+ * @stbuf: Memory location to write stat buffer
+ * @cancellable:
+ * @error:
+ *
+ * Some streams created via libgsystem are #GUnixInputStream; these do
+ * not support e.g. g_file_input_stream_query_info().  This function
+ * allows dropping to the raw unix fstat() call for these types of
+ * streams, while still conveniently wrapped with the normal GLib
+ * handling of @cancellable and @error.
+ */
+gboolean
+gs_stream_fstat (GFileDescriptorBased *stream,
+                 struct stat          *stbuf,
+                 GCancellable         *cancellable,
+                 GError              **error)
+{
+  gboolean ret = FALSE;
+  int fd;
+
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    goto out;
+
+  fd = g_file_descriptor_based_get_fd (stream);
+
+  if (fstat (fd, stbuf) == -1)
+    {
+      _set_error_from_errno (error);
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  return ret;
+}
+
 /**
  * gs_file_map_noatime: (skip)
  * @file: a #GFile
@@ -156,12 +226,8 @@ gs_file_map_noatime (GFile         *file,
   if (path == NULL)
     return NULL;
 
-  fd = _open_fd_noatime (path);
-  if (fd < 0)
-    {
-      _set_error_from_errno (error);
-      return NULL;
-    }
+  if (!gs_file_openat_noatime (AT_FDCWD, path, &fd, cancellable, error))
+    return NULL;
   
   ret = g_mapped_file_new_from_fd (fd, FALSE, error);
   close_nointr_noerror (fd); /* Ignore errors - we always want to close */
@@ -220,12 +286,9 @@ gs_file_sync_data (GFile          *file,
   int res;
   int fd = -1; 
 
-  fd = _open_fd_noatime (gs_file_get_path_cached (file));
-  if (fd < 0)
-    {
-      _set_error_from_errno (error);
-      goto out;
-    }
+  if (!gs_file_openat_noatime (AT_FDCWD, gs_file_get_path_cached (file), &fd,
+                               cancellable, error))
+    goto out;
 
   do
     res = fdatasync (fd);
@@ -279,6 +342,12 @@ gs_file_create (GFile          *file,
       _set_error_from_errno (error);
       goto out;
     }
+
+  if (fchmod (fd, mode) < 0)
+    {
+      _set_error_from_errno (error);
+      goto out;
+    }
   
   ret_stream = g_unix_output_stream_new (fd, TRUE);
   
@@ -299,12 +368,24 @@ get_default_tmp_prefix (void)
       const char *prgname = g_get_prgname ();
       const char *p;
       char *prefix;
+      char *iter;
 
-      p = strrchr (prgname, '/');
-      if (p)
-        prgname = p + 1;
-
+      if (prgname)
+        {
+          p = strrchr (prgname, '/');
+          if (p)
+            prgname = p + 1;
+        }
+      else
+        prgname = "";
+          
       prefix = g_strdup_printf ("tmp-%s%u-", prgname, getuid ());
+      for (iter = prefix; *iter; iter++)
+        {
+          char c = *iter;
+          if (c == ' ')
+            *iter = '_';
+        }
       
       g_once_init_leave (&tmpprefix, prefix);
     }
@@ -313,7 +394,7 @@ get_default_tmp_prefix (void)
 }
 
 /**
- * gsystem_fileutil_gen_tmp_name:
+ * gs_fileutil_gen_tmp_name:
  * @prefix: (allow-none): String prepended to the result
  * @suffix: (allow-none): String suffixed to the result
  *
@@ -322,8 +403,8 @@ get_default_tmp_prefix (void)
  * name does not exist.
  */
 char *
-gsystem_fileutil_gen_tmp_name (const char *prefix,
-                               const char *suffix)
+gs_fileutil_gen_tmp_name (const char *prefix,
+                          const char *suffix)
 {
   static const char table[] = "ABCEDEFGHIJKLMNOPQRSTUVWXYZabcedefghijklmnopqrstuvwxyz0123456789";
   GString *str = g_string_new ("");
@@ -346,6 +427,145 @@ gsystem_fileutil_gen_tmp_name (const char *prefix,
   return g_string_free (str, FALSE);
 }
 
+/**
+ * gs_file_open_dir_fd:
+ * @path: Directory name
+ * @out_fd: (out): File descriptor for directory
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * On success, sets @out_fd to a file descriptor for the directory
+ * that can be used with UNIX functions such as openat().
+ */
+gboolean
+gs_file_open_dir_fd (GFile         *path,
+                     int           *out_fd,
+                     GCancellable  *cancellable,
+                     GError       **error)
+{
+  /* Linux specific probably */
+  *out_fd = open (gs_file_get_path_cached (path), O_RDONLY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+  if (*out_fd == -1)
+    {
+      _set_error_from_errno (error);
+      return FALSE;
+    }
+  return TRUE;
+}
+
+/**
+ * gs_file_open_in_tmpdir_at:
+ * @tmpdir_fd: Directory to place temporary file
+ * @mode: Default mode (will be affected by umask)
+ * @out_name: (out) (transfer full): Newly created file name
+ * @out_stream: (out) (transfer full) (allow-none): Newly created output stream
+ * @cancellable:
+ * @error:
+ *
+ * Like g_file_open_tmp(), except the file will be created in the
+ * provided @tmpdir, and allows specification of the Unix @mode, which
+ * means private files may be created.  Return values will be stored
+ * in @out_name, and optionally @out_stream.
+ */
+gboolean
+gs_file_open_in_tmpdir_at (int                tmpdir_fd,
+                           int                mode,
+                           char             **out_name,
+                           GOutputStream    **out_stream,
+                           GCancellable      *cancellable,
+                           GError           **error)
+{
+  gboolean ret = FALSE;
+  const int max_attempts = 128;
+  guint i;
+  char *tmp_name = NULL;
+  int fd;
+
+  /* 128 attempts seems reasonable... */
+  for (i = 0; i < max_attempts; i++)
+    {
+      g_free (tmp_name);
+      tmp_name = gs_fileutil_gen_tmp_name (NULL, NULL);
+
+      do
+        fd = openat (tmpdir_fd, tmp_name, O_WRONLY | O_CREAT | O_EXCL, mode);
+      while (fd == -1 && errno == EINTR);
+      if (fd < 0 && errno != EEXIST)
+        {
+          _set_error_from_errno (error);
+          goto out;
+        }
+      break;
+    }
+  if (i == max_attempts)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Exhausted attempts to open temporary file");
+      goto out;
+    }
+
+  ret = TRUE;
+  gs_transfer_out_value (out_name, &tmp_name);
+  if (out_stream)
+    *out_stream = g_unix_output_stream_new (fd, TRUE);
+  else
+    (void) close (fd);
+ out:
+  g_free (tmp_name);
+  return ret;
+}
+
+/**
+ * gs_file_open_in_tmpdir:
+ * @tmpdir: Directory to place temporary file
+ * @mode: Default mode (will be affected by umask)
+ * @out_file: (out) (transfer full): Newly created file path
+ * @out_stream: (out) (transfer full) (allow-none): Newly created output stream
+ * @cancellable:
+ * @error:
+ *
+ * Like g_file_open_tmp(), except the file will be created in the
+ * provided @tmpdir, and allows specification of the Unix @mode, which
+ * means private files may be created.  Return values will be stored
+ * in @out_file, and optionally @out_stream.
+ */
+gboolean
+gs_file_open_in_tmpdir (GFile             *tmpdir,
+                        int                mode,
+                        GFile            **out_file,
+                        GOutputStream    **out_stream,
+                        GCancellable      *cancellable,
+                        GError           **error)
+{
+  gboolean ret = FALSE;
+  DIR *d = NULL;
+  int dfd = -1;
+  char *tmp_name = NULL;
+  GOutputStream *ret_stream = NULL;
+
+  d = opendir (gs_file_get_path_cached (tmpdir));
+  if (!d)
+    {
+      _set_error_from_errno (error);
+      goto out;
+    }
+  dfd = dirfd (d);
+
+  if (!gs_file_open_in_tmpdir_at (dfd, mode, &tmp_name,
+                                  out_stream ? &ret_stream : NULL,
+                                  cancellable, error))
+    goto out;
+ 
+  ret = TRUE;
+  *out_file = g_file_get_child (tmpdir, tmp_name);
+  gs_transfer_out_value (out_stream, &ret_stream);
+ out:
+  if (d) (void) closedir (d);
+  g_clear_object (&ret_stream);
+  g_free (tmp_name);
+  return ret;
+}
+
 static gboolean
 linkcopy_internal_attempt (GFile          *src,
                           GFile          *dest,
@@ -365,7 +585,7 @@ linkcopy_internal_attempt (GFile          *src,
   if (g_cancellable_set_error_if_cancelled (cancellable, error))
     goto out;
 
-  tmp_name = gsystem_fileutil_gen_tmp_name (NULL, NULL);
+  tmp_name = gs_fileutil_gen_tmp_name (NULL, NULL);
   tmp_dest = g_file_get_child (dest_parent, tmp_name);
 
   res = link (gs_file_get_path_cached (src), gs_file_get_path_cached (tmp_dest));
@@ -544,6 +764,25 @@ gs_file_linkcopy_sync_data (GFile          *src,
   return linkcopy_internal (src, dest, flags, TRUE, cancellable, error);
 }
 
+static char *
+gs_file_get_target_path (GFile *file)
+{
+  GFileInfo *info;
+  const char *target;
+  char *path;
+
+  info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI, G_FILE_QUERY_INFO_NONE, NULL, NULL);
+  if (info == NULL)
+    return NULL;
+  target = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI);
+  path = g_filename_from_uri (target, NULL, NULL);
+  g_object_unref (info);
+
+  return path;
+}
+
+G_LOCK_DEFINE_STATIC (pathname_cache);
+
 /**
  * gs_file_get_path_cached:
  *
@@ -559,13 +798,26 @@ gs_file_get_path_cached (GFile *file)
   if (G_UNLIKELY (_file_path_quark) == 0)
     _file_path_quark = g_quark_from_static_string ("gsystem-file-path");
 
+  G_LOCK (pathname_cache);
+
   path = g_object_get_qdata ((GObject*)file, _file_path_quark);
   if (!path)
     {
-      path = g_file_get_path (file);
-      g_assert (path != NULL);
+      if (g_file_has_uri_scheme (file, "trash") ||
+          g_file_has_uri_scheme (file, "recent"))
+        path = gs_file_get_target_path (file);
+      else
+        path = g_file_get_path (file);
+      if (path == NULL)
+        {
+          G_UNLOCK (pathname_cache);
+          return NULL;
+        }
       g_object_set_qdata_full ((GObject*)file, _file_path_quark, (char*)path, (GDestroyNotify)g_free);
     }
+
+  G_UNLOCK (pathname_cache);
+
   return path;
 }
 
@@ -584,12 +836,17 @@ gs_file_get_basename_cached (GFile *file)
   if (G_UNLIKELY (_file_name_quark) == 0)
     _file_name_quark = g_quark_from_static_string ("gsystem-file-name");
 
+  G_LOCK (pathname_cache);
+
   name = g_object_get_qdata ((GObject*)file, _file_name_quark);
   if (!name)
     {
       name = g_file_get_basename (file);
       g_object_set_qdata_full ((GObject*)file, _file_name_quark, (char*)name, (GDestroyNotify)g_free);
     }
+
+  G_UNLOCK (pathname_cache);
+
   return name;
 }
 
@@ -745,6 +1002,38 @@ gs_file_unlink (GFile          *path,
   return TRUE;
 }
 
+static gboolean
+chown_internal (GFile          *path,
+                gboolean        dereference_links,
+                guint32         owner,
+                guint32         group,
+                GCancellable   *cancellable,
+                GError        **error)
+{
+  gboolean ret = FALSE;
+  int res;
+
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    return FALSE;
+
+  do
+    if (dereference_links)
+      res = chown (gs_file_get_path_cached (path), owner, group);
+    else
+      res = lchown (gs_file_get_path_cached (path), owner, group);
+  while (G_UNLIKELY (res != 0 && errno == EINTR));
+
+  if (res < 0)
+    {
+      _set_error_from_errno (error);
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  return ret;
+}
+
 /**
  * gs_file_chown:
  * @path: Path to file
@@ -764,25 +1053,29 @@ gs_file_chown (GFile          *path,
                GCancellable   *cancellable,
                GError        **error)
 {
-  gboolean ret = FALSE;
-  int res;
+  return chown_internal (path, TRUE, owner, group, cancellable, error);
+}
 
-  if (g_cancellable_set_error_if_cancelled (cancellable, error))
-    return FALSE;
-
-  do
-    res = chown (gs_file_get_path_cached (path), owner, group);
-  while (G_UNLIKELY (res != 0 && errno == EINTR));
-
-  if (res < 0)
-    {
-      _set_error_from_errno (error);
-      goto out;
-    }
-
-  ret = TRUE;
- out:
-  return ret;
+/**
+ * gs_file_lchown:
+ * @path: Path to file
+ * @owner: UNIX owner
+ * @group: UNIX group
+ * @cancellable: a #GCancellable
+ * @error: a #GError
+ *
+ * Merely wraps UNIX lchown().
+ *
+ * Returns: %TRUE on success, %FALSE on error
+ */
+gboolean
+gs_file_lchown (GFile          *path,
+                guint32         owner,
+                guint32         group,
+                GCancellable   *cancellable,
+                GError        **error)
+{
+  return chown_internal (path, FALSE, owner, group, cancellable, error);
 }
 
 /**
@@ -982,7 +1275,7 @@ gs_file_get_relpath (GFile *one,
   gchar *one_path, *one_suffix;
   gchar *two_path, *two_suffix;
   GString *path;
-  int index;
+  int i;
 
   simple_path = g_file_get_relative_path (one, two);
   if (simple_path)
@@ -991,9 +1284,9 @@ gs_file_get_relpath (GFile *one,
   one_path = g_file_get_path (one);
   two_path = g_file_get_path (two);
 
-  index = path_common_directory (one_path, two_path);
-  one_suffix = one_path + index;
-  two_suffix = two_path + index;
+  i = path_common_directory (one_path, two_path);
+  one_suffix = one_path + i;
+  two_suffix = two_path + i;
 
   path = g_string_new ("");
 
@@ -1025,7 +1318,7 @@ gs_file_get_relpath (GFile *one,
  * followed. That is, it's a #GFile whose path is the result
  * of calling realpath() on @file.
  *
- * Returns: (transfer full): A new #GFile
+ * Returns: (allow-none) (transfer full): A new #GFile or %NULL if @file is invalid
  */
 GFile *
 gs_file_realpath (GFile *file)
@@ -1034,7 +1327,13 @@ gs_file_realpath (GFile *file)
   gchar path_real[PATH_MAX];
 
   path = g_file_get_path (file);
-  realpath ((const char *) path, path_real);
+
+  if (realpath ((const char *) path, path_real) == NULL)
+    {
+      g_free (path);
+      return NULL;
+    }
+
   g_free (path);
   return g_file_new_for_path (path_real);
 }
