@@ -22,6 +22,7 @@
 #define __GSYSTEM_FILE_UTILS_H__
 
 #include <gio/gio.h>
+#include <sys/stat.h>
 
 G_BEGIN_DECLS
 
@@ -35,6 +36,12 @@ gboolean gs_file_enumerator_iterate (GFileEnumerator  *direnum,
                                      GCancellable     *cancellable,
                                      GError          **error);
 
+gboolean gs_file_openat_noatime (int            dfd,
+                                 const char    *name,
+                                 int           *ret_fd,
+                                 GCancellable  *cancellable,
+                                 GError       **error);
+
 GInputStream *gs_file_read_noatime (GFile         *file,
                                     GCancellable  *cancellable,
                                     GError       **error);
@@ -42,6 +49,14 @@ GMappedFile *gs_file_map_noatime (GFile         *file,
                                   GCancellable  *cancellable,
                                   GError       **error);
 
+#ifndef __GI_SCANNER__
+gboolean gs_stream_fstat (GFileDescriptorBased *stream,
+                          struct stat          *out_stbuf,
+                          GCancellable         *cancellable,
+                          GError              **error);
+
+#endif
+
 #if GLIB_CHECK_VERSION(2,34,0)
 GBytes *gs_file_map_readonly (GFile         *file,
                               GCancellable  *cancellable,
@@ -52,8 +67,27 @@ gboolean gs_file_sync_data (GFile          *file,
                             GCancellable   *cancellable,
                             GError        **error);
 
-char * gsystem_fileutil_gen_tmp_name (const char *prefix,
-                                      const char *suffix);
+char * gs_fileutil_gen_tmp_name (const char *prefix,
+                                 const char *suffix);
+
+gboolean gs_file_open_dir_fd (GFile         *path,
+                              int           *out_fd,
+                              GCancellable  *cancellable,
+                              GError       **error);
+
+gboolean gs_file_open_in_tmpdir_at (int                tmpdir_fd,
+                                    int                mode,
+                                    char             **out_name,
+                                    GOutputStream    **out_stream,
+                                    GCancellable      *cancellable,
+                                    GError           **error);
+
+gboolean gs_file_open_in_tmpdir (GFile             *tmpdir,
+                                 int                mode,
+                                 GFile            **out_file,
+                                 GOutputStream    **out_stream,
+                                 GCancellable      *cancellable,
+                                 GError           **error);
 
 gboolean gs_file_create (GFile          *file,
                          int             mode,
@@ -88,6 +122,12 @@ gboolean gs_file_chown (GFile          *path,
                         GCancellable   *cancellable,
                         GError        **error);
 
+gboolean gs_file_lchown (GFile          *path,
+                         guint32         owner,
+                         guint32         group,
+                         GCancellable   *cancellable,
+                         GError        **error);
+
 gboolean gs_file_chmod (GFile          *path,
                         guint           mode,
                         GCancellable   *cancellable,
