@@ -28,6 +28,7 @@
 #include <gio/gfiledescriptorbased.h>
 #include <attr/xattr.h>
 #include "ostree.h"
+#include "ostree-core-private.h"
 #include "ostree-chain-input-stream.h"
 #include "otutil.h"
 #include "libgsystem.h"
@@ -35,6 +36,17 @@
 #define ALIGN_VALUE(this, boundary) \
   (( ((unsigned long)(this)) + (((unsigned long)(boundary)) -1)) & (~(((unsigned long)(boundary))-1)))
 
+static gboolean
+file_header_parse (GVariant         *metadata,
+                   GFileInfo       **out_file_info,
+                   GVariant        **out_xattrs,
+                   GError          **error);
+static gboolean
+zlib_file_header_parse (GVariant         *metadata,
+                        GFileInfo       **out_file_info,
+                        GVariant        **out_xattrs,
+                        GError          **error);
+
 /**
  * SECTION:libostree-core
  * @title: Core repository-independent functions
@@ -163,7 +175,6 @@ ostree_validate_rev (const char *rev,
                      GError **error)
 {
   gboolean ret = FALSE;
-  gs_unref_ptrarray GPtrArray *components = NULL;
   GMatchInfo *match = NULL;
 
   static gsize regex_initialized;
@@ -335,9 +346,9 @@ ostree_get_xattrs_for_file (GFile         *f,
   return ret;
 }
 
-GVariant *
-ostree_file_header_new (GFileInfo         *file_info,
-                        GVariant          *xattrs)
+static GVariant *
+file_header_new (GFileInfo         *file_info,
+                 GVariant          *xattrs)
 {
   guint32 uid;
   guint32 gid;
@@ -367,7 +378,7 @@ ostree_file_header_new (GFileInfo         *file_info,
   return ret;
 }
 
-/**
+/*
  * ostree_zlib_file_header_new:
  * @file_info: a #GFileInfo
  * @xattrs: (allow-none): Optional extended attribute array
@@ -375,8 +386,8 @@ ostree_file_header_new (GFileInfo         *file_info,
  * Returns: (transfer full): A new #GVariant containing file header for an archive-z2 repository
  */
 GVariant *
-ostree_zlib_file_header_new (GFileInfo         *file_info,
-                             GVariant          *xattrs)
+_ostree_zlib_file_header_new (GFileInfo         *file_info,
+                              GVariant          *xattrs)
 {
   guint64 size;
   guint32 uid;
@@ -442,8 +453,8 @@ write_padding (GOutputStream    *output,
   return ret;
 }
 
-/**
- * ostree_write_variant_with_size:
+/*
+ * _ostree_write_variant_with_size:
  * @output: Stream
  * @variant: A variant
  * @alignment_offset: Used to determine whether or not we should write padding bytes
@@ -458,13 +469,13 @@ write_padding (GOutputStream    *output,
  * accessed.
  */
 gboolean
-ostree_write_variant_with_size (GOutputStream      *output,
-                                GVariant           *variant,
-                                guint64             alignment_offset,
-                                gsize              *out_bytes_written,
-                                GChecksum          *checksum,
-                                GCancellable       *cancellable,
-                                GError            **error)
+_ostree_write_variant_with_size (GOutputStream      *output,
+                                 GVariant           *variant,
+                                 guint64             alignment_offset,
+                                 gsize              *out_bytes_written,
+                                 GChecksum          *checksum,
+                                 GCancellable       *cancellable,
+                                 GError            **error)
 {
   gboolean ret = FALSE;
   guint64 variant_size;
@@ -506,8 +517,8 @@ ostree_write_variant_with_size (GOutputStream      *output,
   return ret;
 }
 
-/**
- * ostree_write_file_header_update_checksum:
+/*
+ * write_file_header_update_checksum:
  * @out: Stream
  * @variant: A variant, should be a file header
  * @checksum: (allow-none): If provided, update with written data
@@ -517,18 +528,18 @@ ostree_write_variant_with_size (GOutputStream      *output,
  * Write a file header variant to the provided @out stream, optionally
  * updating @checksum.
  */
-gboolean
-ostree_write_file_header_update_checksum (GOutputStream         *out,
-                                          GVariant              *header,
-                                          GChecksum             *checksum,
-                                          GCancellable          *cancellable,
-                                          GError               **error)
+static gboolean
+write_file_header_update_checksum (GOutputStream         *out,
+                                   GVariant              *header,
+                                   GChecksum             *checksum,
+                                   GCancellable          *cancellable,
+                                   GError               **error)
 {
   gboolean ret = FALSE;
   gsize bytes_written;
 
-  if (!ostree_write_variant_with_size (out, header, 0, &bytes_written, checksum,
-                                       cancellable, error))
+  if (!_ostree_write_variant_with_size (out, header, 0, &bytes_written, checksum,
+                                        cancellable, error))
     goto out;
 
   ret = TRUE;
@@ -568,12 +579,12 @@ ostree_raw_file_to_content_stream (GInputStream       *input,
   gs_unref_object GOutputStream *header_out_stream = NULL;
   gs_unref_object GInputStream *header_in_stream = NULL;
 
-  file_header = ostree_file_header_new (file_info, xattrs);
+  file_header = file_header_new (file_info, xattrs);
 
   header_out_stream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
 
-  if (!ostree_write_variant_with_size (header_out_stream, file_header, 0, NULL, NULL,
-                                       cancellable, error))
+  if (!_ostree_write_variant_with_size (header_out_stream, file_header, 0, NULL, NULL,
+                                        cancellable, error))
     goto out;
 
   if (!g_output_stream_close (header_out_stream, cancellable, error))
@@ -630,7 +641,6 @@ ostree_content_stream_parse (gboolean                compressed,
   guchar dummy[4];
   gsize bytes_read;
   gs_unref_object GInputStream *ret_input = NULL;
-  gs_unref_object GFileInfo *content_file_info = NULL;
   gs_unref_object GFileInfo *ret_file_info = NULL;
   gs_unref_variant GVariant *ret_xattrs = NULL;
   gs_unref_variant GVariant *file_header = NULL;
@@ -665,25 +675,25 @@ ostree_content_stream_parse (gboolean                compressed,
   if (!g_input_stream_read_all (input, buf, archive_header_size, &bytes_read,
                                 cancellable, error))
     goto out;
-  file_header = g_variant_new_from_data (compressed ? OSTREE_ZLIB_FILE_HEADER_GVARIANT_FORMAT : OSTREE_FILE_HEADER_GVARIANT_FORMAT,
+  file_header = g_variant_new_from_data (compressed ? _OSTREE_ZLIB_FILE_HEADER_GVARIANT_FORMAT : _OSTREE_FILE_HEADER_GVARIANT_FORMAT,
                                          buf, archive_header_size, trusted,
                                          g_free, buf);
   buf = NULL;
 
   if (compressed)
     {
-      if (!ostree_zlib_file_header_parse (file_header,
-                                          out_file_info ? &ret_file_info : NULL,
-                                          out_xattrs ? &ret_xattrs : NULL,
-                                          error))
+      if (!zlib_file_header_parse (file_header,
+                                   out_file_info ? &ret_file_info : NULL,
+                                   out_xattrs ? &ret_xattrs : NULL,
+                                   error))
         goto out;
     }
   else
     {
-      if (!ostree_file_header_parse (file_header,
-                                     out_file_info ? &ret_file_info : NULL,
-                                     out_xattrs ? &ret_xattrs : NULL,
-                                     error))
+      if (!file_header_parse (file_header,
+                              out_file_info ? &ret_file_info : NULL,
+                              out_xattrs ? &ret_xattrs : NULL,
+                              error))
         goto out;
       if (ret_file_info)
         g_file_info_set_size (ret_file_info, input_length - archive_header_size - 8);
@@ -829,10 +839,10 @@ ostree_checksum_file_from_input (GFileInfo        *file_info,
     {
       gs_unref_variant GVariant *file_header = NULL;
 
-      file_header = ostree_file_header_new (file_info, xattrs);
+      file_header = file_header_new (file_info, xattrs);
 
-      if (!ostree_write_file_header_update_checksum (NULL, file_header, checksum,
-                                                     cancellable, error))
+      if (!write_file_header_update_checksum (NULL, file_header, checksum,
+                                              cancellable, error))
         goto out;
 
       if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_REGULAR)
@@ -1026,8 +1036,45 @@ ostree_create_directory_metadata (GFileInfo    *dir_info,
   return ret_metadata;
 }
 
-/**
- * ostree_set_xattrs:
+gboolean
+_ostree_set_xattrs_fd (int            fd,
+                       GVariant      *xattrs,
+                       GCancellable  *cancellable,
+                       GError       **error)
+{
+  gboolean ret = FALSE;
+  int i, n;
+
+  n = g_variant_n_children (xattrs);
+  for (i = 0; i < n; i++)
+    {
+      const guint8* name;
+      gs_unref_variant GVariant *value = NULL;
+      const guint8* value_data;
+      gsize value_len;
+      int res;
+
+      g_variant_get_child (xattrs, i, "(^&ay@ay)",
+                           &name, &value);
+      value_data = g_variant_get_fixed_array (value, &value_len, 1);
+      
+      do
+        res = fsetxattr (fd, (char*)name, (char*)value_data, value_len, 0);
+      while (G_UNLIKELY (res == -1 && errno == EINTR));
+      if (G_UNLIKELY (res == -1))
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+    }
+
+  ret = TRUE;
+ out:
+  return ret;
+}
+
+/*
+ * _ostree_set_xattrs:
  * @f: a file
  * @xattrs: Extended attribute list
  * @cancellable: Cancellable
@@ -1038,10 +1085,10 @@ ostree_create_directory_metadata (GFileInfo    *dir_info,
  * attributes.
  */
 gboolean
-ostree_set_xattrs (GFile  *f, 
-                   GVariant *xattrs, 
-                   GCancellable *cancellable, 
-                   GError **error)
+_ostree_set_xattrs (GFile  *f, 
+                    GVariant *xattrs, 
+                    GCancellable *cancellable, 
+                    GError **error)
 {
   const char *path;
   gboolean ret = FALSE;
@@ -1077,6 +1124,62 @@ ostree_set_xattrs (GFile  *f,
   return ret;
 }
 
+/* Create a randomly-named symbolic link in @tempdir which points to
+ * @target.  The filename will be returned in @out_file.
+ *
+ * The reason this odd function exists is that the repo should only
+ * contain objects in their final state.  For bare repositories, we
+ * need to first create the symlink, then chown it, and apply all
+ * extended attributes, before finally rename()ing it into place.
+ *
+ * Furthermore for checkouts, we use this to implement union mode
+ * where we override existing files via tempfile+rename().
+ */
+gboolean
+_ostree_make_temporary_symlink_at (int             tmp_dirfd,
+                                   const char     *target,
+                                   char          **out_name,
+                                   GCancellable   *cancellable,
+                                   GError        **error)
+{
+  gboolean ret = FALSE;
+  gs_free char *tmpname = NULL;
+  guint i;
+  const int max_attempts = 128;
+
+  for (i = 0; i < max_attempts; i++)
+    {
+      g_free (tmpname);
+      tmpname = gs_fileutil_gen_tmp_name (NULL, NULL);
+      if (symlinkat (target, tmp_dirfd, tmpname) < 0)
+        {
+          if (errno == EEXIST)
+            continue;
+          else
+            {
+              int errsv = errno;
+              g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errsv),
+                                   g_strerror (errsv));
+              goto out;
+            }
+        }
+      else
+        break;
+    }
+  if (i == max_attempts)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Exhausted attempts to open temporary file");
+      goto out;
+    }
+
+  ret = TRUE;
+  gs_transfer_out_value (out_name, &tmpname);
+ out:
+  return ret;
+}
+
+
 /**
  * ostree_object_type_to_string:
  * @objtype: an #OstreeObjectType
@@ -1348,8 +1451,54 @@ ostree_checksum_bytes_peek (GVariant *bytes)
   return g_variant_get_fixed_array (bytes, &n_elts, 1);
 }
 
-/**
- * ostree_get_relative_object_path:
+/*
+ * _ostree_loose_path:
+ * @buf: Output buffer, must be _OSTREE_LOOSE_PATH_MAX in size
+ * @checksum: ASCII checksum
+ * @objtype: Object type
+ * @mode: Repository mode
+ *
+ * Overwrite the contents of @buf with relative path for loose
+ * object.
+ */
+void
+_ostree_loose_path (char              *buf,
+                    const char        *checksum,
+                    OstreeObjectType   objtype,
+                    OstreeRepoMode     mode)
+{
+  _ostree_loose_path_with_suffix (buf, checksum, objtype, mode, "");
+}
+
+/*
+ * _ostree_loose_path_with_suffix:
+ * @buf: Output buffer, must be _OSTREE_LOOSE_PATH_MAX in size
+ * @checksum: ASCII checksum
+ * @objtype: Object type
+ * @mode: Repository mode
+ *
+ * Like _ostree_loose_path, but also append a further arbitrary
+ * suffix; useful for finding non-core objects.
+ */
+void
+_ostree_loose_path_with_suffix (char              *buf,
+                                const char        *checksum,
+                                OstreeObjectType   objtype,
+                                OstreeRepoMode     mode,
+                                const char        *suffix)
+{
+  *buf = checksum[0];
+  buf++;
+  *buf = checksum[1];
+  buf++;
+  snprintf (buf, _OSTREE_LOOSE_PATH_MAX - 2, "/%s.%s%s%s",
+            checksum + 2, ostree_object_type_to_string (objtype),
+            (!OSTREE_OBJECT_TYPE_IS_META (objtype) && mode == OSTREE_REPO_MODE_ARCHIVE_Z2) ? "z" : "",
+            suffix);
+}
+
+/*
+ * _ostree_get_relative_object_path:
  * @checksum: ASCII checksum string
  * @type: Object type
  * @compressed: Whether or not the repository object is compressed
@@ -1357,9 +1506,9 @@ ostree_checksum_bytes_peek (GVariant *bytes)
  * Returns: (transfer full): Relative path for a loose object
  */
 char *
-ostree_get_relative_object_path (const char         *checksum,
-                                 OstreeObjectType    type,
-                                 gboolean            compressed)
+_ostree_get_relative_object_path (const char         *checksum,
+                                  OstreeObjectType    type,
+                                  gboolean            compressed)
 {
   GString *path;
 
@@ -1378,7 +1527,7 @@ ostree_get_relative_object_path (const char         *checksum,
   return g_string_free (path, FALSE);
 }
 
-/**
+/*
  * ostree_file_header_parse:
  * @metadata: A metadata variant of type %OSTREE_FILE_HEADER_GVARIANT_FORMAT
  * @out_file_info: (out): Parsed file information
@@ -1388,11 +1537,11 @@ ostree_get_relative_object_path (const char         *checksum,
  * Load file header information into standard Gio #GFileInfo object,
  * along with extended attributes tored in @out_xattrs.
  */
-gboolean
-ostree_file_header_parse (GVariant         *metadata,
-                          GFileInfo       **out_file_info,
-                          GVariant        **out_xattrs,
-                          GError          **error)
+static gboolean
+file_header_parse (GVariant         *metadata,
+                   GFileInfo       **out_file_info,
+                   GVariant        **out_xattrs,
+                   GError          **error)
 {
   gboolean ret = FALSE;
   guint32 uid, gid, mode, rdev;
@@ -1438,8 +1587,8 @@ ostree_file_header_parse (GVariant         *metadata,
   return ret;
 }
 
-/**
- * ostree_zlib_file_header_parse:
+/*
+ * zlib_file_header_parse:
  * @metadata: A metadata variant of type %OSTREE_FILE_HEADER_GVARIANT_FORMAT
  * @out_file_info: (out): Parsed file information
  * @out_xattrs: (out): Parsed extended attribute set
@@ -1448,11 +1597,11 @@ ostree_file_header_parse (GVariant         *metadata,
  * Like ostree_file_header_parse(), but operates on zlib-compressed
  * content.
  */
-gboolean
-ostree_zlib_file_header_parse (GVariant         *metadata,
-                               GFileInfo       **out_file_info,
-                               GVariant        **out_xattrs,
-                               GError          **error)
+static gboolean
+zlib_file_header_parse (GVariant         *metadata,
+                        GFileInfo       **out_file_info,
+                        GVariant        **out_xattrs,
+                        GError          **error)
 {
   gboolean ret = FALSE;
   guint64 size;
@@ -1501,259 +1650,6 @@ ostree_zlib_file_header_parse (GVariant         *metadata,
   return ret;
 }
 
-/**
- * ostree_create_file_from_input:
- * @dest_file: Destination; must not exist
- * @finfo: File information
- * @xattrs: (allow-none): Optional extended attributes
- * @input: (allow-none): Optional file content, must be %NULL for symbolic links
- * @cancellable: Cancellable
- * @error: Error
- *
- * Create a directory, regular file, or symbolic link, based on
- * @finfo.  Append extended attributes from @xattrs if provided.  For
- * %G_FILE_TYPE_REGULAR, set content based on @input.
- */
-gboolean
-ostree_create_file_from_input (GFile            *dest_file,
-                               GFileInfo        *finfo,
-                               GVariant         *xattrs,
-                               GInputStream     *input,
-                               GCancellable     *cancellable,
-                               GError          **error)
-{
-  gboolean ret = FALSE;
-  const char *dest_path;
-  guint32 uid, gid, mode;
-  gs_unref_object GOutputStream *out = NULL;
-
-  if (g_cancellable_set_error_if_cancelled (cancellable, error))
-    return FALSE;
-
-  if (finfo != NULL)
-    {
-      mode = g_file_info_get_attribute_uint32 (finfo, "unix::mode");
-    }
-  else
-    {
-      mode = S_IFREG | 0664;
-    }
-  dest_path = gs_file_get_path_cached (dest_file);
-
-  if (S_ISDIR (mode))
-    {
-      if (mkdir (gs_file_get_path_cached (dest_file), mode) < 0)
-        {
-          ot_util_set_error_from_errno (error, errno);
-          goto out;
-        }
-    }
-  else if (S_ISREG (mode))
-    {
-      if (finfo != NULL)
-        {
-          uid = g_file_info_get_attribute_uint32 (finfo, "unix::uid");
-          gid = g_file_info_get_attribute_uint32 (finfo, "unix::gid");
-
-          if (!gs_file_create_with_uidgid (dest_file, mode, uid, gid, &out,
-                                           cancellable, error))
-            goto out;
-        }
-      else
-        {
-          if (!gs_file_create (dest_file, mode, &out,
-                               cancellable, error))
-            goto out;
-        }
-
-      if (input)
-        {
-          if (g_output_stream_splice ((GOutputStream*)out, input, 0,
-                                      cancellable, error) < 0)
-            goto out;
-        }
-
-      if (!g_output_stream_close ((GOutputStream*)out, NULL, error))
-        goto out;
-
-      /* Work around libguestfs/FUSE bug */
-      if (mode & (S_ISUID|S_ISGID))
-        {
-          if (chmod (dest_path, mode) == -1)
-            {
-              ot_util_set_error_from_errno (error, errno);
-              goto out;
-            }
-        }
-    }
-  else if (S_ISLNK (mode))
-    {
-      const char *target = g_file_info_get_attribute_byte_string (finfo, "standard::symlink-target");
-      if (symlink (target, dest_path) < 0)
-        {
-          ot_util_set_error_from_errno (error, errno);
-          goto out;
-        }
-    }
-  else
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Invalid mode %u", mode);
-      goto out;
-    }
-
-  /* We only need to chown for directories and symlinks; we already
-   * did a chown for files above via fchown().
-   */
-  if (finfo != NULL && !S_ISREG (mode))
-    {
-      uid = g_file_info_get_attribute_uint32 (finfo, "unix::uid");
-      gid = g_file_info_get_attribute_uint32 (finfo, "unix::gid");
-      
-      if (lchown (dest_path, uid, gid) < 0)
-        {
-          ot_util_set_error_from_errno (error, errno);
-          g_prefix_error (error, "lchown(%u, %u) failed: ", uid, gid);
-          goto out;
-        }
-    }
-
-  if (xattrs != NULL)
-    {
-      if (!ostree_set_xattrs (dest_file, xattrs, cancellable, error))
-        goto out;
-    }
-
-  ret = TRUE;
- out:
-  if (!ret && !S_ISDIR(mode))
-    {
-      (void) unlink (dest_path);
-    }
-  return ret;
-}
-
-/**
- * ostree_create_temp_file_from_input:
- * @dir: Target directory
- * @prefix: Optional prefix
- * @suffix: Optional suffix
- * @finfo: File information
- * @xattrs: (allow-none): Optional extended attributes
- * @input: (allow-none): Optional file content, must be %NULL for symbolic links
- * @out_file: (out): Path for newly created directory, file, or symbolic link
- * @cancellable: Cancellable
- * @error: Error
- *
- * Like ostree_create_file_from_input(), but securely allocates a
- * randomly-named target in @dir.  This is a unified version of
- * mkstemp()/mkdtemp() that also supports symbolic links.
- */
-gboolean
-ostree_create_temp_file_from_input (GFile            *dir,
-                                    const char       *prefix,
-                                    const char       *suffix,
-                                    GFileInfo        *finfo,
-                                    GVariant         *xattrs,
-                                    GInputStream     *input,
-                                    GFile           **out_file,
-                                    GCancellable     *cancellable,
-                                    GError          **error)
-{
-  gboolean ret = FALSE;
-  GError *temp_error = NULL;
-  int i = 0;
-  gs_unref_object GFile *possible_file = NULL;
-  gs_free guchar *ret_csum = NULL;
-
-  /* 128 attempts seems reasonable... */
-  for (i = 0; i < 128; i++)
-    {
-      gs_free char *possible_name = NULL;
-
-      if (g_cancellable_set_error_if_cancelled (cancellable, error))
-        goto out;
-
-      possible_name = gsystem_fileutil_gen_tmp_name (prefix, suffix);
-      g_clear_object (&possible_file);
-      possible_file = g_file_get_child (dir, possible_name);
-      
-      if (!ostree_create_file_from_input (possible_file, finfo, xattrs, input,
-                                          cancellable, &temp_error))
-        {
-          if (g_error_matches (temp_error, G_IO_ERROR, G_IO_ERROR_EXISTS))
-            {
-              g_clear_error (&temp_error);
-              continue;
-            }
-          else
-            {
-              g_propagate_error (error, temp_error);
-              goto out;
-            }
-        }
-      else
-        {
-          break;
-        }
-    }
-  if (i >= 128)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Exhausted 128 attempts to create a temporary file");
-      goto out;
-    }
-
-  ret = TRUE;
-  ot_transfer_out_value(out_file, &possible_file);
- out:
-  return ret;
-}
-
-/**
- * ostree_create_temp_dir:
- * @dir: Use this as temporary base
- * @prefix: (allow-none): Optional prefix
- * @suffix: (allow-none): Optional suffix
- * @out_file: (out): Path for newly created directory, file, or symbolic link
- * @cancellable: Cancellable
- * @error: Error
- *
- * Securely create a randomly-named temporary subdirectory of @dir.
- */
-gboolean
-ostree_create_temp_dir (GFile            *dir,
-                        const char       *prefix,
-                        const char       *suffix,
-                        GFile           **out_file,
-                        GCancellable     *cancellable,
-                        GError          **error)
-{
-  gboolean ret = FALSE;
-  gs_free char *template = NULL;
-  gs_unref_object GFile *ret_file = NULL;
-
-  if (dir == NULL)
-    dir = g_file_new_for_path (g_get_tmp_dir ());
-
-  template = g_strdup_printf ("%s/%s-XXXXXX",
-                              gs_file_get_path_cached (dir),
-                              prefix ? prefix : "tmp");
-  
-  if (mkdtemp (template) == NULL)
-    {
-      ot_util_set_error_from_errno (error, errno);
-      goto out;
-    }
-
-  ret_file = g_file_new_for_path (template);
-
-  ret = TRUE;
-  ot_transfer_out_value (out_file, &ret_file);
- out:
-  return ret;
-}
-
 /**
  * ostree_validate_structureof_objtype:
  * @objtype:
