@@ -72,26 +72,6 @@ typedef enum {
  */
 #define OSTREE_OBJECT_TYPE_LAST OSTREE_OBJECT_TYPE_COMMIT
 
-/**
- * OSTREE_FILE_HEADER_GVARIANT_FORMAT:
- *
- * File objects are stored as a stream, with one #GVariant header,
- * followed by content.
- * 
- * The file header is of the following form:
- *
- * &lt;BE guint32 containing variant length&gt;
- * u - uid
- * u - gid
- * u - mode
- * u - rdev
- * s - symlink target 
- * a(ayay) - xattrs
- *
- * Then the rest of the stream is data.
- */
-#define OSTREE_FILE_HEADER_GVARIANT_FORMAT G_VARIANT_TYPE ("(uuuusa(ayay))")
-
 /**
  * OSTREE_DIRMETA_GVARIANT_FORMAT:
  *
@@ -125,23 +105,17 @@ typedef enum {
 #define OSTREE_COMMIT_GVARIANT_FORMAT G_VARIANT_TYPE ("(a{sv}aya(say)sstayay)")
 
 /**
- * OSTREE_ZLIB_FILE_HEADER_GVARIANT_FORMAT:
+ * OstreeRepoMode:
+ * @OSTREE_REPO_MODE_BARE: Files are stored as themselves; can only be written as root
+ * @OSTREE_REPO_MODE_ARCHIVE_Z2: Files are compressed, should be owned by non-root.  Can be served via HTTP
  *
- * This is a variation on %OSTREE_FILE_HEADER_GVARIANT_FORMAT, used for
- * storing zlib-compressed content objects.
- *
- * &lt;BE guint32 containing variant length&gt;
- * t - size
- * u - uid
- * u - gid
- * u - mode
- * u - rdev
- * s - symlink target 
- * a(ayay) - xattrs
- * ---
- * zlib-compressed data
+ * See the documentation of #OstreeRepo for more information about the
+ * possible modes.
  */
-#define OSTREE_ZLIB_FILE_HEADER_GVARIANT_FORMAT G_VARIANT_TYPE ("(tuuuusa(ayay))")
+typedef enum {
+  OSTREE_REPO_MODE_BARE,
+  OSTREE_REPO_MODE_ARCHIVE_Z2
+} OstreeRepoMode;
 
 const GVariantType *ostree_metadata_variant_type (OstreeObjectType objtype);
 
@@ -193,45 +167,11 @@ void ostree_object_from_string (const char *str,
                                 gchar     **out_checksum,
                                 OstreeObjectType *out_objtype);
 
-char *ostree_get_relative_object_path (const char        *checksum,
-                                       OstreeObjectType   type,
-                                       gboolean           compressed);
-
 gboolean ostree_get_xattrs_for_file (GFile         *f,
                                      GVariant     **out_xattrs,
                                      GCancellable  *cancellable,
                                      GError       **error);
 
-gboolean ostree_set_xattrs (GFile *f, GVariant *xattrs,
-                            GCancellable *cancellable, GError **error);
-
-gboolean ostree_map_metadata_file (GFile                       *file,
-                                   OstreeObjectType             expected_type,
-                                   GVariant                   **out_variant,
-                                   GError                     **error);
-
-gboolean ostree_write_variant_with_size (GOutputStream      *output,
-                                         GVariant           *variant,
-                                         guint64             alignment_offset,
-                                         gsize              *out_bytes_written,
-                                         GChecksum          *checksum,
-                                         GCancellable       *cancellable,
-                                         GError            **error);
-
-GVariant *ostree_file_header_new (GFileInfo         *file_info,
-                                  GVariant          *xattrs);
-GVariant *ostree_zlib_file_header_new (GFileInfo         *file_info,
-                                       GVariant          *xattrs);
-
-gboolean ostree_file_header_parse (GVariant         *metadata,
-                                   GFileInfo       **out_file_info,
-                                   GVariant        **out_xattrs,
-                                   GError          **error);
-gboolean ostree_zlib_file_header_parse (GVariant         *metadata,
-                                        GFileInfo       **out_file_info,
-                                        GVariant        **out_xattrs,
-                                        GError          **error);
-
 gboolean
 ostree_content_stream_parse (gboolean                compressed,
                              GInputStream           *input,
@@ -252,12 +192,6 @@ gboolean ostree_content_file_parse (gboolean                compressed,
                                     GCancellable           *cancellable,
                                     GError                **error);
 
-gboolean ostree_write_file_header_update_checksum (GOutputStream         *out,
-                                                   GVariant              *variant,
-                                                   GChecksum             *checksum,
-                                                   GCancellable          *cancellable,
-                                                   GError               **error);
-
 gboolean ostree_raw_file_to_content_stream (GInputStream       *input,
                                             GFileInfo          *file_info,
                                             GVariant           *xattrs,
@@ -295,30 +229,6 @@ gboolean ostree_checksum_file_async_finish (GFile          *f,
 GVariant *ostree_create_directory_metadata (GFileInfo *dir_info,
                                             GVariant  *xattrs);
 
-gboolean ostree_create_file_from_input (GFile          *dest_file,
-                                        GFileInfo      *finfo,
-                                        GVariant       *xattrs,
-                                        GInputStream   *input,
-                                        GCancellable   *cancellable,
-                                        GError        **error);
-
-gboolean ostree_create_temp_file_from_input (GFile            *dir,
-                                             const char       *prefix,
-                                             const char       *suffix,
-                                             GFileInfo        *finfo,
-                                             GVariant         *xattrs,
-                                             GInputStream     *input,
-                                             GFile           **out_file,
-                                             GCancellable     *cancellable,
-                                             GError          **error);
-
-gboolean ostree_create_temp_dir (GFile            *dir,
-                                 const char       *prefix,
-                                 const char       *suffix,
-                                 GFile           **out_file,
-                                 GCancellable     *cancellable,
-                                 GError          **error);
-
 /** VALIDATION **/
 
 gboolean ostree_validate_structureof_objtype (guchar    objtype,
