@@ -56,6 +56,7 @@
 #include "config.h"
 
 #include "ostree.h"
+#include "ostree-core-private.h"
 #include "ostree-repo-private.h"
 #include "ostree-fetcher.h"
 #include "otutil.h"
@@ -65,6 +66,7 @@ typedef struct {
     PULL_MSG_SCAN_IDLE,
     PULL_MSG_MAIN_IDLE,
     PULL_MSG_FETCH,
+    PULL_MSG_FETCH_DETACHED_METADATA,
     PULL_MSG_SCAN,
     PULL_MSG_QUIT
   } t;
@@ -88,7 +90,9 @@ typedef struct {
 
   gboolean      transaction_resuming;
   volatile gint n_scanned_metadata;
-  guint         outstanding_uri_requests;
+  SoupURI       *fetching_sync_uri;
+  
+  gboolean          gpg_verify;
 
   GThread          *metadata_thread;
   GMainContext     *metadata_thread_context;
@@ -101,9 +105,9 @@ typedef struct {
   guint             metadata_scan_idle : 1; /* TRUE if we passed through an idle message */
   guint             idle_serial; /* Incremented when we get a SCAN_IDLE message */
   guint             n_outstanding_metadata_fetches;
-  guint             n_outstanding_metadata_stage_requests;
+  guint             n_outstanding_metadata_write_requests;
   guint             n_outstanding_content_fetches;
-  guint             n_outstanding_content_stage_requests;
+  guint             n_outstanding_content_write_requests;
   gint              n_requested_metadata;
   gint              n_requested_content;
   guint             n_fetched_metadata;
@@ -121,6 +125,7 @@ typedef struct {
   OtPullData  *pull_data;
   GVariant    *object;
   GFile       *temp_path;
+  gboolean     is_detached_meta;
 } FetchObjectData;
 
 static SoupURI *
@@ -177,17 +182,21 @@ static gboolean
 uri_fetch_update_status (gpointer user_data)
 {
   OtPullData *pull_data = user_data;
-  gs_free char *fetcher_status = NULL;
   GString *status;
-  guint outstanding_stages;
+  guint outstanding_writes;
   guint outstanding_fetches;
  
   status = g_string_new ("");
 
   outstanding_fetches = pull_data->n_outstanding_content_fetches + pull_data->n_outstanding_metadata_fetches;
-  outstanding_stages = pull_data->n_outstanding_content_stage_requests + pull_data->n_outstanding_metadata_stage_requests;
+  outstanding_writes = pull_data->n_outstanding_content_write_requests + pull_data->n_outstanding_metadata_write_requests;
 
-  if (outstanding_fetches)
+  if (pull_data->fetching_sync_uri)
+    {
+      gs_free char *uri_string = soup_uri_to_string (pull_data->fetching_sync_uri, TRUE);
+      g_string_append_printf (status, "Requesting %s", uri_string);
+    }
+  else if (outstanding_fetches)
     {
       guint64 bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
       guint fetched = pull_data->n_fetched_metadata + pull_data->n_fetched_content;
@@ -200,8 +209,8 @@ uri_fetch_update_status (gpointer user_data)
                               (guint)((((double)fetched) / requested) * 100),
                               fetched, requested, formatted_bytes_transferred);
     }
-  else if (outstanding_stages > 0)
-    g_string_append_printf (status, "Writing objects: %u", outstanding_stages);
+  else if (outstanding_writes > 0)
+    g_string_append_printf (status, "Writing objects: %u", outstanding_writes);
   else if (!pull_data->metadata_scan_idle)
     g_string_append_printf (status, "Scanning metadata: %u",
                             g_atomic_int_get (&pull_data->n_scanned_metadata));
@@ -228,6 +237,7 @@ pull_worker_message_new (int msgtype, gpointer data)
       break;
     case PULL_MSG_SCAN:
     case PULL_MSG_FETCH:
+    case PULL_MSG_FETCH_DETACHED_METADATA:
       msg->d.item = data;
       break;
     case PULL_MSG_QUIT:
@@ -261,22 +271,22 @@ check_outstanding_requests_handle_error (OtPullData          *pull_data,
 {
   gboolean current_fetch_idle = (pull_data->n_outstanding_metadata_fetches == 0 &&
                                  pull_data->n_outstanding_content_fetches == 0);
-  gboolean current_stage_idle = (pull_data->n_outstanding_metadata_stage_requests == 0 &&
-                                 pull_data->n_outstanding_content_stage_requests == 0);
+  gboolean current_write_idle = (pull_data->n_outstanding_metadata_write_requests == 0 &&
+                                 pull_data->n_outstanding_content_write_requests == 0);
 
   g_debug ("pull: scan: %u fetching: %u staging: %u",
-           !pull_data->metadata_scan_idle, !current_fetch_idle, !current_stage_idle);
+           !pull_data->metadata_scan_idle, !current_fetch_idle, !current_write_idle);
 
   throw_async_error (pull_data, error);
 
   /* This is true in the phase when we're fetching refs */
   if (pull_data->metadata_objects_to_scan == NULL)
     {
-      if (pull_data->outstanding_uri_requests == 0)
+      if (!pull_data->fetching_sync_uri)
         g_main_loop_quit (pull_data->loop);
       return;
     }
-  else if (pull_data->metadata_scan_idle && current_fetch_idle && current_stage_idle)
+  else if (pull_data->metadata_scan_idle && current_fetch_idle && current_write_idle)
     {
       g_main_loop_quit (pull_data->loop);
     }
@@ -324,74 +334,63 @@ run_mainloop_monitor_fetcher (OtPullData   *pull_data)
 
 typedef struct {
   OtPullData     *pull_data;
-  GFile          *result_file;
-} OstreeFetchUriData;
+  GInputStream   *result_stream;
+} OstreeFetchUriSyncData;
 
 static void
-uri_fetch_on_complete (GObject        *object,
-                       GAsyncResult   *result,
-                       gpointer        user_data) 
+fetch_uri_sync_on_complete (GObject        *object,
+                            GAsyncResult   *result,
+                            gpointer        user_data) 
 {
-  OstreeFetchUriData *data = user_data;
-  GError *local_error = NULL;
+  OstreeFetchUriSyncData *data = user_data;
 
-  data->result_file = ostree_fetcher_request_uri_finish ((OstreeFetcher*)object,
-                                                         result, &local_error);
-  data->pull_data->outstanding_uri_requests--;
-  check_outstanding_requests_handle_error (data->pull_data, local_error);
+  data->result_stream = ostree_fetcher_stream_uri_finish ((OstreeFetcher*)object,
+                                                          result, data->pull_data->async_error);
+  data->pull_data->fetching_sync_uri = NULL;
+  g_main_loop_quit (data->pull_data->loop);
 }
 
 static gboolean
-fetch_uri (OtPullData  *pull_data,
-           SoupURI     *uri,
-           const char  *tmp_prefix,
-           GFile      **out_temp_filename,
-           GCancellable  *cancellable,
-           GError     **error)
+fetch_uri_contents_utf8_sync (OtPullData  *pull_data,
+                              SoupURI     *uri,
+                              char       **out_contents,
+                              GCancellable  *cancellable,
+                              GError     **error)
 {
   gboolean ret = FALSE;
-  gs_free char *uri_string = NULL;
-  gs_unref_object SoupRequest *request = NULL;
-  OstreeFetchUriData fetch_data = { 0, };
+  const guint8 nulchar = 0;
+  gs_free char *ret_contents = NULL;
+  gs_unref_object GMemoryOutputStream *buf = NULL;
+  OstreeFetchUriSyncData fetch_data = { 0, };
 
   if (g_cancellable_set_error_if_cancelled (cancellable, error))
     return FALSE;
 
   fetch_data.pull_data = pull_data;
 
-  uri_string = soup_uri_to_string (uri, FALSE);
+  pull_data->fetching_sync_uri = uri;
+  ostree_fetcher_stream_uri_async (pull_data->fetcher, uri, cancellable,
+                                   fetch_uri_sync_on_complete, &fetch_data);
 
-  pull_data->outstanding_uri_requests++;
-  ostree_fetcher_request_uri_async (pull_data->fetcher, uri, cancellable,
-                                    uri_fetch_on_complete, &fetch_data);
-
-  if (!run_mainloop_monitor_fetcher (pull_data))
+  run_mainloop_monitor_fetcher (pull_data);
+  if (!fetch_data.result_stream)
     goto out;
 
-  ret = TRUE;
-  ot_transfer_out_value (out_temp_filename, &fetch_data.result_file);
- out:
-  return ret;
-}
-
-static gboolean
-fetch_uri_contents_utf8 (OtPullData  *pull_data,
-                         SoupURI     *uri,
-                         char       **out_contents,
-                         GCancellable  *cancellable,
-                         GError     **error)
-{
-  gboolean ret = FALSE;
-  gs_unref_object GFile *tmpf = NULL;
-  gs_free char *ret_contents = NULL;
-  gsize len;
-
-  if (!fetch_uri (pull_data, uri, "tmp-", &tmpf, cancellable, error))
+  buf = (GMemoryOutputStream*)g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
+  if (g_output_stream_splice ((GOutputStream*)buf, fetch_data.result_stream,
+                              G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE,
+                              cancellable, error) < 0)
     goto out;
 
-  if (!g_file_load_contents (tmpf, cancellable, &ret_contents, &len, NULL, error))
+  /* Add trailing NUL */
+  if (!g_output_stream_write ((GOutputStream*)buf, &nulchar, 1, cancellable, error))
     goto out;
 
+  if (!g_output_stream_close ((GOutputStream*)buf, cancellable, error))
+    goto out;
+
+  ret_contents = g_memory_output_stream_steal_data (buf);
+
   if (!g_utf8_validate (ret_contents, -1, NULL))
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
@@ -402,8 +401,7 @@ fetch_uri_contents_utf8 (OtPullData  *pull_data,
   ret = TRUE;
   ot_transfer_out_value (out_contents, &ret_contents);
  out:
-  if (tmpf)
-    (void) unlink (gs_file_get_path_cached (tmpf));
+  g_clear_object (&(fetch_data.result_stream));
   return ret;
 }
 
@@ -419,7 +417,6 @@ scan_dirtree_object (OtPullData   *pull_data,
   gs_unref_variant GVariant *tree = NULL;
   gs_unref_variant GVariant *files_variant = NULL;
   gs_unref_variant GVariant *dirs_variant = NULL;
-  gs_unref_object GFile *stored_path = NULL;
 
   if (recursion_depth > OSTREE_MAX_RECURSION)
     {
@@ -472,7 +469,6 @@ scan_dirtree_object (OtPullData   *pull_data,
       const char *dirname;
       gs_unref_variant GVariant *tree_csum = NULL;
       gs_unref_variant GVariant *meta_csum = NULL;
-      gs_free char *tmp_checksum = NULL;
 
       g_variant_get_child (dirs_variant, i, "(&s@ay@ay)",
                            &dirname, &tree_csum, &meta_csum);
@@ -509,7 +505,7 @@ fetch_ref_contents (OtPullData    *pull_data,
 
   target_uri = suburi_new (pull_data->base_uri, "refs", "heads", ref, NULL);
   
-  if (!fetch_uri_contents_utf8 (pull_data, target_uri, &ret_contents, cancellable, error))
+  if (!fetch_uri_contents_utf8_sync (pull_data, target_uri, &ret_contents, cancellable, error))
     goto out;
 
   g_strchomp (ret_contents);
@@ -526,7 +522,7 @@ fetch_ref_contents (OtPullData    *pull_data,
 }
 
 static void
-content_fetch_on_stage_complete (GObject        *object,
+content_fetch_on_write_complete (GObject        *object,
                                  GAsyncResult   *result,
                                  gpointer        user_data)
 {
@@ -539,7 +535,7 @@ content_fetch_on_stage_complete (GObject        *object,
   gs_free guchar *csum = NULL;
   gs_free char *checksum = NULL;
 
-  if (!ostree_repo_stage_content_finish ((OstreeRepo*)object, result, 
+  if (!ostree_repo_write_content_finish ((OstreeRepo*)object, result, 
                                          &csum, error))
     goto out;
 
@@ -548,7 +544,7 @@ content_fetch_on_stage_complete (GObject        *object,
   ostree_object_name_deserialize (fetch_data->object, &expected_checksum, &objtype);
   g_assert (objtype == OSTREE_OBJECT_TYPE_FILE);
 
-  g_debug ("stage of %s complete", ostree_object_to_string (checksum, objtype));
+  g_debug ("write of %s complete", ostree_object_to_string (checksum, objtype));
 
   if (strcmp (checksum, expected_checksum) != 0)
     {
@@ -560,7 +556,7 @@ content_fetch_on_stage_complete (GObject        *object,
 
   pull_data->n_fetched_content++;
  out:
-  pull_data->n_outstanding_content_stage_requests--;
+  pull_data->n_outstanding_content_write_requests--;
   check_outstanding_requests_handle_error (pull_data, local_error);
   (void) gs_file_unlink (fetch_data->temp_path, NULL, NULL);
   g_object_unref (fetch_data->temp_path);
@@ -579,17 +575,14 @@ content_fetch_on_complete (GObject        *object,
   GError **error = &local_error;
   GCancellable *cancellable = NULL;
   guint64 length;
-  gs_unref_variant GVariant *file_meta = NULL;
   gs_unref_object GFileInfo *file_info = NULL;
-  gs_unref_object GInputStream *content_input = NULL;
-  gs_unref_object GInputStream *file_object_input = NULL;
   gs_unref_variant GVariant *xattrs = NULL;
   gs_unref_object GInputStream *file_in = NULL;
   gs_unref_object GInputStream *object_input = NULL;
   const char *checksum;
   OstreeObjectType objtype;
 
-  fetch_data->temp_path = ostree_fetcher_request_uri_finish ((OstreeFetcher*)object, result, error);
+  fetch_data->temp_path = ostree_fetcher_request_uri_with_partial_finish ((OstreeFetcher*)object, result, error);
   if (!fetch_data->temp_path)
     goto out;
 
@@ -608,11 +601,11 @@ content_fetch_on_complete (GObject        *object,
                                           cancellable, error))
     goto out;
   
-  pull_data->n_outstanding_content_stage_requests++;
-  ostree_repo_stage_content_async (pull_data->repo, checksum,
+  pull_data->n_outstanding_content_write_requests++;
+  ostree_repo_write_content_async (pull_data->repo, checksum,
                                    object_input, length,
                                    cancellable,
-                                   content_fetch_on_stage_complete, fetch_data);
+                                   content_fetch_on_write_complete, fetch_data);
 
  out:
   pull_data->n_outstanding_content_fetches--;
@@ -620,7 +613,7 @@ content_fetch_on_complete (GObject        *object,
 }
 
 static void
-on_metadata_staged (GObject           *object,
+on_metadata_writed (GObject           *object,
                     GAsyncResult      *result,
                     gpointer           user_data)
 {
@@ -633,7 +626,7 @@ on_metadata_staged (GObject           *object,
   gs_free char *checksum = NULL;
   gs_free guchar *csum = NULL;
 
-  if (!ostree_repo_stage_metadata_finish ((OstreeRepo*)object, result, 
+  if (!ostree_repo_write_metadata_finish ((OstreeRepo*)object, result, 
                                           &csum, error))
     goto out;
 
@@ -642,7 +635,7 @@ on_metadata_staged (GObject           *object,
   ostree_object_name_deserialize (fetch_data->object, &expected_checksum, &objtype);
   g_assert (OSTREE_OBJECT_TYPE_IS_META (objtype));
 
-  g_debug ("stage of %s complete", ostree_object_to_string (checksum, objtype));
+  g_debug ("write of %s complete", ostree_object_to_string (checksum, objtype));
 
   if (strcmp (checksum, expected_checksum) != 0)
     {
@@ -657,7 +650,7 @@ on_metadata_staged (GObject           *object,
                           pull_worker_message_new (PULL_MSG_SCAN,
                                                   g_variant_ref (fetch_data->object)));
  out:
-  pull_data->n_outstanding_metadata_stage_requests--;
+  pull_data->n_outstanding_metadata_write_requests--;
   (void) gs_file_unlink (fetch_data->temp_path, NULL, NULL);
   g_object_unref (fetch_data->temp_path);
   g_variant_unref (fetch_data->object);
@@ -666,6 +659,11 @@ on_metadata_staged (GObject           *object,
   check_outstanding_requests_handle_error (pull_data, local_error);
 }
 
+static void
+enqueue_one_object_request (OtPullData        *pull_data,
+                            GVariant          *object_name,
+                            gboolean           is_detached_meta);
+
 static void
 meta_fetch_on_complete (GObject           *object,
                         GAsyncResult      *result,
@@ -679,23 +677,47 @@ meta_fetch_on_complete (GObject           *object,
   GError *local_error = NULL;
   GError **error = &local_error;
 
-  fetch_data->temp_path = ostree_fetcher_request_uri_finish ((OstreeFetcher*)object, result, error);
-  if (!fetch_data->temp_path)
-    goto out;
-
   ostree_object_name_deserialize (fetch_data->object, &checksum, &objtype);
-
   g_debug ("fetch of %s complete", ostree_object_to_string (checksum, objtype));
 
-  if (!ot_util_variant_map (fetch_data->temp_path, ostree_metadata_variant_type (objtype),
-                            FALSE, &metadata, error))
-    goto out;
+  fetch_data->temp_path = ostree_fetcher_request_uri_with_partial_finish ((OstreeFetcher*)object, result, error);
+  if (!fetch_data->temp_path)
+    {
+      if (!g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+        goto out;
+      else if (fetch_data->is_detached_meta)
+        {
+          /* There isn't any detached metadata, just fetch the commit */
+          g_clear_error (&local_error);
+          enqueue_one_object_request (pull_data, fetch_data->object, FALSE);
+        }
 
-  ostree_repo_stage_metadata_async (pull_data->repo, objtype, checksum, metadata,
-                                    pull_data->cancellable,
-                                    on_metadata_staged, fetch_data);
+      goto out;
+    }
+
+  if (fetch_data->is_detached_meta)
+    {
+      if (!ot_util_variant_map (fetch_data->temp_path, G_VARIANT_TYPE ("a{sv}"),
+                                FALSE, &metadata, error))
+        goto out;
+      if (!ostree_repo_write_commit_detached_metadata (pull_data->repo, checksum, metadata,
+                                                       pull_data->cancellable, error))
+        goto out;
+
+      enqueue_one_object_request (pull_data, fetch_data->object, FALSE);
+    }
+  else
+    {
+      if (!ot_util_variant_map (fetch_data->temp_path, ostree_metadata_variant_type (objtype),
+                                FALSE, &metadata, error))
+        goto out;
+      
+      ostree_repo_write_metadata_async (pull_data->repo, objtype, checksum, metadata,
+                                        pull_data->cancellable,
+                                        on_metadata_writed, fetch_data);
+      pull_data->n_outstanding_metadata_write_requests++;
+    }
 
-  pull_data->n_outstanding_metadata_stage_requests++;
  out:
   pull_data->n_outstanding_metadata_fetches--;
   pull_data->n_fetched_metadata++;
@@ -716,10 +738,8 @@ scan_commit_object (OtPullData         *pull_data,
 {
   gboolean ret = FALSE;
   gs_unref_variant GVariant *commit = NULL;
-  gs_unref_variant GVariant *related_objects = NULL;
   gs_unref_variant GVariant *tree_contents_csum = NULL;
   gs_unref_variant GVariant *tree_meta_csum = NULL;
-  gs_free char *tmp_checksum = NULL;
   GVariantIter *iter = NULL;
 
   if (recursion_depth > OSTREE_MAX_RECURSION)
@@ -729,6 +749,19 @@ scan_commit_object (OtPullData         *pull_data,
       goto out;
     }
 
+#ifdef HAVE_GPGME
+  if (pull_data->gpg_verify)
+    {
+      if (!ostree_repo_verify_commit (pull_data->repo,
+                                      checksum,
+                                      NULL,
+                                      NULL,
+                                      cancellable,
+                                      error))
+        goto out;
+    }
+#endif
+
   if (!ostree_repo_load_variant (pull_data->repo, OSTREE_OBJECT_TYPE_COMMIT, checksum,
                                  &commit, error))
     goto out;
@@ -784,9 +817,14 @@ scan_one_metadata_object (OtPullData         *pull_data,
       char *duped_checksum = g_strdup (tmp_checksum);
       g_hash_table_insert (pull_data->requested_metadata, duped_checksum, duped_checksum);
       
-      ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
-                              pull_worker_message_new (PULL_MSG_FETCH,
-                                                       g_variant_ref (object)));
+      if (objtype == OSTREE_OBJECT_TYPE_COMMIT)
+        ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
+                                pull_worker_message_new (PULL_MSG_FETCH_DETACHED_METADATA,
+                                                         g_variant_ref (object)));
+      else
+        ot_waitable_queue_push (pull_data->metadata_objects_to_fetch,
+                                pull_worker_message_new (PULL_MSG_FETCH,
+                                                         g_variant_ref (object)));
     }
   else if (is_stored)
     {
@@ -933,6 +971,53 @@ metadata_thread_main (gpointer user_data)
   return NULL;
 }
 
+static void
+enqueue_one_object_request (OtPullData        *pull_data,
+                            GVariant          *object_name,
+                            gboolean           is_detached_meta)
+{
+  const char *checksum;
+  OstreeObjectType objtype;
+  SoupURI *obj_uri = NULL;
+  gboolean is_meta;
+  FetchObjectData *fetch_data;
+  gs_free char *objpath = NULL;
+
+  ostree_object_name_deserialize (object_name, &checksum, &objtype);
+
+  if (is_detached_meta)
+    {
+      char buf[_OSTREE_LOOSE_PATH_MAX];
+      _ostree_loose_path_with_suffix (buf, checksum, OSTREE_OBJECT_TYPE_COMMIT,
+                                      pull_data->remote_mode, "meta");
+      obj_uri = suburi_new (pull_data->base_uri, "objects", buf, NULL);
+    }
+  else
+    {
+      objpath = _ostree_get_relative_object_path (checksum, objtype, TRUE);
+      obj_uri = suburi_new (pull_data->base_uri, objpath, NULL);
+    }
+
+  is_meta = OSTREE_OBJECT_TYPE_IS_META (objtype);
+  if (is_meta)
+    {
+      pull_data->n_outstanding_metadata_fetches++;
+      pull_data->n_requested_metadata++;
+    }
+  else
+    {
+      pull_data->n_outstanding_content_fetches++;
+      pull_data->n_requested_content++;
+    }
+  fetch_data = g_new0 (FetchObjectData, 1);
+  fetch_data->pull_data = pull_data;
+  fetch_data->object = g_variant_ref (object_name);
+  fetch_data->is_detached_meta = is_detached_meta;
+  ostree_fetcher_request_uri_with_partial_async (pull_data->fetcher, obj_uri, pull_data->cancellable,
+                                                 is_meta ? meta_fetch_on_complete : content_fetch_on_complete, fetch_data);
+  soup_uri_free (obj_uri);
+}
+
 static gboolean
 on_metadata_objects_to_fetch_ready (gint         fd,
                                     GIOCondition condition,
@@ -963,36 +1048,14 @@ on_metadata_objects_to_fetch_ready (gint         fd,
                                   pull_worker_message_new (PULL_MSG_MAIN_IDLE, GUINT_TO_POINTER (pull_data->idle_serial)));
         }
     }
-  else if (msg->t == PULL_MSG_FETCH)
+  else if (msg->t == PULL_MSG_FETCH || msg->t == PULL_MSG_FETCH_DETACHED_METADATA)
     {
-      const char *checksum;
-      gs_free char *objpath = NULL;
-      OstreeObjectType objtype;
-      SoupURI *obj_uri = NULL;
-      gboolean is_meta;
-      FetchObjectData *fetch_data;
+      gboolean is_detached_meta;
+
+      is_detached_meta = msg->t == PULL_MSG_FETCH_DETACHED_METADATA;
       
-      ostree_object_name_deserialize (msg->d.item, &checksum, &objtype);
-      objpath = ostree_get_relative_object_path (checksum, objtype, TRUE);
-      obj_uri = suburi_new (pull_data->base_uri, objpath, NULL);
+      enqueue_one_object_request (pull_data, msg->d.item, is_detached_meta);
 
-      is_meta = OSTREE_OBJECT_TYPE_IS_META (objtype);
-      if (is_meta)
-        {
-          pull_data->n_outstanding_metadata_fetches++;
-          pull_data->n_requested_metadata++;
-        }
-      else
-        {
-          pull_data->n_outstanding_content_fetches++;
-          pull_data->n_requested_content++;
-        }
-      fetch_data = g_new (FetchObjectData, 1);
-      fetch_data->pull_data = pull_data;
-      fetch_data->object = g_variant_ref (msg->d.item);
-      ostree_fetcher_request_uri_async (pull_data->fetcher, obj_uri, pull_data->cancellable,
-                                        is_meta ? meta_fetch_on_complete : content_fetch_on_complete, fetch_data);
-      soup_uri_free (obj_uri);
       g_variant_unref (msg->d.item);
     }
   else
@@ -1113,8 +1176,8 @@ load_remote_repo_config (OtPullData    *pull_data,
 
   target_uri = suburi_new (pull_data->base_uri, "config", NULL);
   
-  if (!fetch_uri_contents_utf8 (pull_data, target_uri, &contents,
-                                cancellable, error))
+  if (!fetch_uri_contents_utf8_sync (pull_data, target_uri, &contents,
+                                     cancellable, error))
     goto out;
 
   ret_keyfile = g_key_file_new ();
@@ -1144,14 +1207,12 @@ ostree_repo_pull (OstreeRepo               *self,
   gboolean tls_permissive = FALSE;
   OstreeFetcherConfigFlags fetcher_flags = 0;
   gs_free char *remote_key = NULL;
-  gs_free char *remote_config_content = NULL;
   gs_free char *path = NULL;
   gs_free char *baseurl = NULL;
   gs_free char *summary_data = NULL;
   gs_unref_hashtable GHashTable *requested_refs_to_fetch = NULL;
   gs_unref_hashtable GHashTable *updated_refs = NULL;
   gs_unref_hashtable GHashTable *commits_to_fetch = NULL;
-  gs_free char *branch_rev = NULL;
   gs_free char *remote_mode_str = NULL;
   GSource *queue_src = NULL;
   OtPullData pull_data_real = { 0, };
@@ -1165,7 +1226,7 @@ ostree_repo_pull (OstreeRepo               *self,
   guint64 end_time;
 
   pull_data->async_error = error;
-  pull_data->main_context = g_main_context_get_thread_default ();
+  pull_data->main_context = g_main_context_ref_thread_default ();
   pull_data->loop = g_main_loop_new (pull_data->main_context, FALSE);
   pull_data->flags = flags;
 
@@ -1188,6 +1249,14 @@ ostree_repo_pull (OstreeRepo               *self,
     goto out;
   pull_data->base_uri = soup_uri_new (baseurl);
 
+#ifdef HAVE_GPGME
+  if (!ot_keyfile_get_boolean_with_default (config, remote_key, "gpg-verify",
+                                            TRUE, &pull_data->gpg_verify, error))
+    goto out;
+#else
+  pull_data->gpg_verify = FALSE;
+#endif
+
   if (!ot_keyfile_get_boolean_with_default (config, remote_key, "tls-permissive",
                                             FALSE, &tls_permissive, error))
     goto out;
@@ -1277,7 +1346,7 @@ ostree_repo_pull (OstreeRepo               *self,
           path = g_build_filename (soup_uri_get_path (summary_uri), "refs", "summary", NULL);
           soup_uri_set_path (summary_uri, path);
           
-          if (!fetch_uri_contents_utf8 (pull_data, summary_uri, &summary_data, cancellable, error))
+          if (!fetch_uri_contents_utf8_sync (pull_data, summary_uri, &summary_data, cancellable, error))
             goto out;
           
           if (!parse_ref_summary (summary_data, &requested_refs_to_fetch, error))
@@ -1303,7 +1372,7 @@ ostree_repo_pull (OstreeRepo               *self,
         }
     }
 
-  if (!ostree_repo_prepare_transaction (pull_data->repo, FALSE, &pull_data->transaction_resuming,
+  if (!ostree_repo_prepare_transaction (pull_data->repo, &pull_data->transaction_resuming,
                                         cancellable, error))
     goto out;
 
@@ -1326,9 +1395,6 @@ ostree_repo_pull (OstreeRepo               *self,
     {
       const char *ref = key;
       const char *sha256 = value;
-      gs_free char *key = NULL;
-      gs_free char *remote_ref = NULL;
-      gs_free char *baseurl = NULL;
 
       ot_waitable_queue_push (pull_data->metadata_objects_to_scan,
                               pull_worker_message_new (PULL_MSG_SCAN,
@@ -1352,8 +1418,6 @@ ostree_repo_pull (OstreeRepo               *self,
   if (!run_mainloop_monitor_fetcher (pull_data))
     goto out;
   
-  if (!ostree_repo_commit_transaction (pull_data->repo, cancellable, error))
-    goto out;
 
   g_hash_table_iter_init (&hash_iter, updated_refs);
   while (g_hash_table_iter_next (&hash_iter, &key, &value))
@@ -1374,13 +1438,15 @@ ostree_repo_pull (OstreeRepo               *self,
         }
       else
         {
-          if (!ostree_repo_write_ref (pull_data->repo, pull_data->remote_name, ref, checksum, error))
-            goto out;
-          
+          ostree_repo_transaction_set_ref (pull_data->repo, pull_data->remote_name, ref, checksum);
+
           g_print ("remote %s is now %s\n", remote_ref, checksum);
         }
     }
-      
+
+  if (!ostree_repo_commit_transaction (pull_data->repo, NULL, cancellable, error))
+    goto out;
+
   end_time = g_get_monotonic_time ();
 
   bytes_transferred = ostree_fetcher_bytes_transferred (pull_data->fetcher);
