@@ -33,33 +33,27 @@ G_BEGIN_DECLS
 #define OSTREE_IS_REPO(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OSTREE_TYPE_REPO))
 
+gboolean ostree_repo_mode_from_string (const char      *mode,
+                                       OstreeRepoMode  *out_mode,
+                                       GError         **error);
+
 GType ostree_repo_get_type (void);
 
 OstreeRepo* ostree_repo_new (GFile *path);
 
 OstreeRepo* ostree_repo_new_default (void);
 
-gboolean      ostree_repo_check (OstreeRepo  *self, GError **error);
+gboolean      ostree_repo_open   (OstreeRepo     *self,
+                                  GCancellable   *cancellable,
+                                  GError        **error);
+
+gboolean      ostree_repo_create (OstreeRepo     *self,
+                                  OstreeRepoMode  mode,
+                                  GCancellable   *cancellable,
+                                  GError        **error);
 
 GFile *       ostree_repo_get_path (OstreeRepo  *self);
 
-/**
- * OstreeRepoMode:
- * @OSTREE_REPO_MODE_BARE: Files are stored as themselves; can only be written as root
- * @OSTREE_REPO_MODE_ARCHIVE_Z2: Files are compressed, should be owned by non-root.  Can be served via HTTP
- *
- * See the documentation of #OstreeRepo for more information about the
- * possible modes.
- */
-typedef enum {
-  OSTREE_REPO_MODE_BARE,
-  OSTREE_REPO_MODE_ARCHIVE_Z2
-} OstreeRepoMode;
-
-gboolean       ostree_repo_mode_from_string (const char      *mode,
-                                             OstreeRepoMode  *out_mode,
-                                             GError         **error);
-
 OstreeRepoMode ostree_repo_get_mode (OstreeRepo  *self);
 
 GKeyFile *    ostree_repo_get_config (OstreeRepo *self);
@@ -72,29 +66,66 @@ gboolean      ostree_repo_write_config (OstreeRepo *self,
                                         GKeyFile   *new_config,
                                         GError    **error);
 
+/**
+ * OstreeRepoTransactionStats:
+ * @metadata_objects_total: The total number of metadata objects
+ * in the repository after this transaction has completed.
+ * @metadata_objects_written: The number of metadata objects that
+ * were written to the repository in this transaction.
+ * @content_objects_total: The total number of content objects
+ * in the repository after this transaction has completed.
+ * @content_objects_written: The number of content objects that
+ * were written to the repository in this transaction.
+ * @content_bytes_total: The amount of data added to the repository,
+ * in bytes, counting only content objects.
+ *
+ * A list of statistics for each transaction that may be
+ * interesting for reporting purposes.
+ */
+typedef struct _OstreeRepoTransactionStats OstreeRepoTransactionStats;
+
+struct _OstreeRepoTransactionStats {
+  guint metadata_objects_total;
+  guint metadata_objects_written;
+  guint content_objects_total;
+  guint content_objects_written;
+  guint64 content_bytes_written;
+
+  guint64 padding1;
+  guint64 padding2;
+  guint64 padding3;
+  guint64 padding4;
+};
+
+GType ostree_repo_transaction_stats_get_type (void);
+
+gboolean      ostree_repo_scan_hardlinks      (OstreeRepo     *self,
+                                               GCancellable   *cancellable,
+                                               GError        **error);
+
 gboolean      ostree_repo_prepare_transaction (OstreeRepo     *self,
-                                               gboolean        enable_commit_hardlink_scan,
                                                gboolean       *out_transaction_resume,
                                                GCancellable   *cancellable,
                                                GError        **error);
 
-gboolean      ostree_repo_commit_transaction (OstreeRepo     *self,
-                                              GCancellable   *cancellable,
-                                              GError        **error);
-
-gboolean      ostree_repo_commit_transaction_with_stats (OstreeRepo     *self,
-                                                         guint          *out_metadata_objects_total,
-                                                         guint          *out_metadata_objects_written,
-                                                         guint          *out_content_objects_total,
-                                                         guint          *out_content_objects_written,
-                                                         guint64        *out_content_bytes_written,
-                                                         GCancellable   *cancellable,
-                                                         GError        **error);
+gboolean      ostree_repo_commit_transaction (OstreeRepo                  *self,
+                                              OstreeRepoTransactionStats  *out_stats,
+                                              GCancellable                *cancellable,
+                                              GError                     **error);
 
 gboolean      ostree_repo_abort_transaction (OstreeRepo     *self,
                                              GCancellable   *cancellable,
                                              GError        **error);
 
+void          ostree_repo_transaction_set_refspec (OstreeRepo *self,
+                                                   const char *refspec,
+                                                   const char *checksum);
+
+void          ostree_repo_transaction_set_ref     (OstreeRepo *self,
+                                                   const char *remote,
+                                                   const char *ref,
+                                                   const char *checksum);
+
 gboolean      ostree_repo_has_object (OstreeRepo           *self,
                                       OstreeObjectType      objtype,
                                       const char           *checksum,
@@ -102,7 +133,7 @@ gboolean      ostree_repo_has_object (OstreeRepo           *self,
                                       GCancellable         *cancellable,
                                       GError              **error);
 
-gboolean      ostree_repo_stage_metadata (OstreeRepo        *self,
+gboolean      ostree_repo_write_metadata (OstreeRepo        *self,
                                           OstreeObjectType   objtype,
                                           const char        *expected_checksum,
                                           GVariant          *object,
@@ -110,7 +141,7 @@ gboolean      ostree_repo_stage_metadata (OstreeRepo        *self,
                                           GCancellable      *cancellable,
                                           GError           **error);
 
-void          ostree_repo_stage_metadata_async (OstreeRepo              *self,
+void          ostree_repo_write_metadata_async (OstreeRepo              *self,
                                                 OstreeObjectType         objtype,
                                                 const char              *expected_checksum,
                                                 GVariant                *object,
@@ -118,12 +149,12 @@ void          ostree_repo_stage_metadata_async (OstreeRepo              *self,
                                                 GAsyncReadyCallback      callback,
                                                 gpointer                 user_data);
 
-gboolean      ostree_repo_stage_metadata_finish (OstreeRepo        *self,
+gboolean      ostree_repo_write_metadata_finish (OstreeRepo        *self,
                                                  GAsyncResult      *result,
                                                  guchar           **out_csum,
                                                  GError           **error);
 
-gboolean      ostree_repo_stage_content (OstreeRepo       *self,
+gboolean      ostree_repo_write_content (OstreeRepo       *self,
                                          const char       *expected_checksum,
                                          GInputStream     *object_input,
                                          guint64           length,
@@ -131,21 +162,29 @@ gboolean      ostree_repo_stage_content (OstreeRepo       *self,
                                          GCancellable     *cancellable,
                                          GError          **error);
 
-gboolean      ostree_repo_stage_metadata_trusted (OstreeRepo        *self,
+gboolean      ostree_repo_write_metadata_trusted (OstreeRepo        *self,
                                                   OstreeObjectType   objtype,
                                                   const char        *checksum,
                                                   GVariant          *variant,
                                                   GCancellable      *cancellable,
                                                   GError           **error);
 
-gboolean      ostree_repo_stage_content_trusted (OstreeRepo       *self,
+gboolean      ostree_repo_write_metadata_stream_trusted (OstreeRepo        *self,
+                                                         OstreeObjectType   objtype,
+                                                         const char        *checksum,
+                                                         GInputStream      *object_input,
+                                                         guint64            length,
+                                                         GCancellable      *cancellable,
+                                                         GError           **error);
+
+gboolean      ostree_repo_write_content_trusted (OstreeRepo       *self,
                                                  const char       *checksum,
                                                  GInputStream     *object_input,
                                                  guint64           length,
                                                  GCancellable     *cancellable,
                                                  GError          **error);
 
-void          ostree_repo_stage_content_async (OstreeRepo              *self,
+void          ostree_repo_write_content_async (OstreeRepo              *self,
                                                const char              *expected_checksum,
                                                GInputStream            *object,
                                                guint64                  length,
@@ -153,7 +192,7 @@ void          ostree_repo_stage_content_async (OstreeRepo              *self,
                                                GAsyncReadyCallback      callback,
                                                gpointer                 user_data);
 
-gboolean      ostree_repo_stage_content_finish (OstreeRepo        *self,
+gboolean      ostree_repo_write_content_finish (OstreeRepo        *self,
                                                 GAsyncResult      *result,
                                                 guchar           **out_csum,
                                                 GError           **error);
@@ -164,29 +203,12 @@ gboolean      ostree_repo_resolve_rev (OstreeRepo  *self,
                                        char       **out_rev,
                                        GError     **error);
 
-gboolean      ostree_repo_write_ref (OstreeRepo  *self,
-                                     const char  *remote,
-                                     const char  *name,
-                                     const char  *rev,
-                                     GError     **error);
-
-gboolean      ostree_repo_write_refspec (OstreeRepo  *self,
-                                         const char  *refspec,
-                                         const char  *rev,
-                                         GError     **error);
-
 gboolean      ostree_repo_list_refs (OstreeRepo       *self,
                                      const char       *refspec_prefix,
                                      GHashTable      **out_all_refs,
                                      GCancellable     *cancellable,
                                      GError          **error);
 
-gboolean      ostree_repo_load_variant_c (OstreeRepo  *self,
-                                          OstreeObjectType objtype,
-                                          const guchar  *csum,       
-                                          GVariant     **out_variant,
-                                          GError       **error);
-
 gboolean      ostree_repo_load_variant (OstreeRepo  *self,
                                         OstreeObjectType objtype,
                                         const char    *sha256, 
@@ -271,44 +293,56 @@ typedef struct OstreeRepoCommitModifier OstreeRepoCommitModifier;
 
 OstreeRepoCommitModifier *ostree_repo_commit_modifier_new (OstreeRepoCommitModifierFlags  flags,
                                                            OstreeRepoCommitFilter         commit_filter,
-                                                           gpointer                       user_data);
+                                                           gpointer                       user_data,
+                                                           GDestroyNotify                 destroy_notify);
 
 GType ostree_repo_commit_modifier_get_type (void);
 
 OstreeRepoCommitModifier *ostree_repo_commit_modifier_ref (OstreeRepoCommitModifier *modifier);
 void ostree_repo_commit_modifier_unref (OstreeRepoCommitModifier *modifier);
 
-gboolean      ostree_repo_stage_directory_to_mtree (OstreeRepo                 *self,
+gboolean      ostree_repo_write_directory_to_mtree (OstreeRepo                 *self,
                                                     GFile                      *dir,
                                                     OstreeMutableTree          *mtree,
                                                     OstreeRepoCommitModifier   *modifier,
                                                     GCancellable               *cancellable,
                                                     GError                    **error);
 
-gboolean      ostree_repo_stage_archive_to_mtree (OstreeRepo                   *self,
+gboolean      ostree_repo_write_archive_to_mtree (OstreeRepo                   *self,
                                                   GFile                        *archive,
-                                                  OstreeMutableTree            *tree,
+                                                  OstreeMutableTree            *mtree,
                                                   OstreeRepoCommitModifier     *modifier,
                                                   gboolean                      autocreate_parents,
                                                   GCancellable                 *cancellable,
                                                   GError                      **error);
 
-gboolean      ostree_repo_stage_mtree (OstreeRepo         *self,
+gboolean      ostree_repo_write_mtree (OstreeRepo         *self,
                                        OstreeMutableTree  *mtree,
-                                       char              **out_contents_checksum,
+                                       GFile             **out_file,
                                        GCancellable       *cancellable,
                                        GError            **error);
 
-gboolean      ostree_repo_stage_commit (OstreeRepo   *self,
-                                        const char   *branch,
-                                        const char   *parent,
-                                        const char   *subject,
-                                        const char   *body,
-                                        const char   *root_contents_checksum,
-                                        const char   *root_metadata_checksum,
-                                        char        **out_commit,
-                                        GCancellable *cancellable,
-                                        GError      **error);
+gboolean      ostree_repo_write_commit (OstreeRepo      *self,
+                                        const char      *parent,
+                                        const char      *subject,
+                                        const char      *body,
+                                        GVariant        *metadata,
+                                        OstreeRepoFile  *root,
+                                        char           **out_commit,
+                                        GCancellable    *cancellable,
+                                        GError         **error);
+
+gboolean      ostree_repo_read_commit_detached_metadata (OstreeRepo      *self,
+                                                         const char      *checksum,
+                                                         GVariant       **out_metadata,
+                                                         GCancellable    *cancellable,
+                                                         GError         **error);
+
+gboolean      ostree_repo_write_commit_detached_metadata (OstreeRepo      *self,
+                                                          const char      *checksum,
+                                                          GVariant        *metadata,
+                                                          GCancellable    *cancellable,
+                                                          GError         **error);
 
 /**
  * OstreeRepoCheckoutMode:
@@ -344,11 +378,12 @@ gboolean       ostree_repo_checkout_gc (OstreeRepo        *self,
                                         GCancellable      *cancellable,
                                         GError           **error);
 
-gboolean       ostree_repo_read_commit (OstreeRepo *self,
-                                        const char *rev,
-                                        GFile       **out_root,
-                                        GCancellable *cancellable,
-                                        GError  **error);
+gboolean       ostree_repo_read_commit (OstreeRepo    *self,
+                                        const char    *ref,
+                                        GFile        **out_root,
+                                        char         **out_commit,
+                                        GCancellable  *cancellable,
+                                        GError        **error);
 
 /**
  * OstreeRepoListObjectsFlags:
@@ -378,19 +413,20 @@ gboolean ostree_repo_list_objects (OstreeRepo                  *self,
 
 GHashTable *ostree_repo_traverse_new_reachable (void);
 
-gboolean ostree_repo_traverse_dirtree (OstreeRepo         *repo,
-                                       const char         *commit_checksum,
-                                       GHashTable         *inout_reachable,
-                                       GCancellable       *cancellable,
-                                       GError            **error);
-
 gboolean ostree_repo_traverse_commit (OstreeRepo         *repo,
                                       const char         *commit_checksum,
                                       int                 maxdepth,
-                                      GHashTable         *inout_reachable,
+                                      GHashTable        **out_reachable,
                                       GCancellable       *cancellable,
                                       GError            **error);
 
+gboolean ostree_repo_traverse_commit_union (OstreeRepo         *repo,
+                                            const char         *commit_checksum,
+                                            int                 maxdepth,
+                                            GHashTable         *inout_reachable,
+                                            GCancellable       *cancellable,
+                                            GError            **error);
+
 /**
  * OstreeRepoPruneFlags:
  * @OSTREE_REPO_PRUNE_FLAGS_NONE: No special options for pruning
@@ -427,5 +463,21 @@ gboolean ostree_repo_pull (OstreeRepo             *self,
                            GCancellable           *cancellable,
                            GError                **error);
 
+#ifdef HAVE_GPGME
+gboolean ostree_repo_sign_commit (OstreeRepo     *self,
+                                  const gchar    *commit_checksum,
+                                  const gchar    *key_id,
+                                  const gchar    *homedir,
+                                  GCancellable   *cancellable,
+                                  GError        **error);
+
+gboolean ostree_repo_verify_commit (OstreeRepo   *self,
+                                    const gchar  *commit_checksum,
+                                    GFile        *keyringdir,
+                                    GFile        *extra_keyring,
+                                    GCancellable *cancellable,
+                                    GError      **error);
+#endif
+
 G_END_DECLS
 
