@@ -26,4 +26,7 @@
 #include <ostree-repo.h>
 #include <ostree-mutable-tree.h>
 #include <ostree-repo-file.h>
+#include <ostree-sysroot.h>
+#include <ostree-deployment.h>
+#include <ostree-bootconfig-parser.h>
 #include <ostree-diff.h>
