@@ -26,6 +26,8 @@
 
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
+#include <locale.h>
 
 #include "ot-main.h"
 #include "ot-builtins.h"
@@ -66,13 +68,23 @@ main (int    argc,
   GError *error = NULL;
   int ret;
 
+  setlocale (LC_ALL, "");
+
   ret = ostree_run (argc, argv, commands, &error);
   if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
     ostree_usage (argv, commands, TRUE);
 
   if (error != NULL)
     {
-      g_message ("%s", error->message);
+      int is_tty = isatty (1);
+      const char *prefix = "";
+      const char *suffix = "";
+      if (is_tty)
+        {
+          prefix = "\x1b[31m\x1b[1m"; /* red, bold */
+          suffix = "\x1b[22m\x1b[0m"; /* bold off, color reset */
+        }
+      g_printerr ("%serror: %s%s\n", prefix, suffix, error->message);
       g_error_free (error);
     }
 
