@@ -56,20 +56,19 @@ process_one_checkout (OstreeRepo           *repo,
 {
   gboolean ret = FALSE;
   GError *tmp_error = NULL;
-  gs_unref_object OstreeRepoFile *root = NULL;
-  gs_unref_object OstreeRepoFile *subtree = NULL;
+  gs_unref_object GFile *root = NULL;
+  gs_unref_object GFile *subtree = NULL;
   gs_unref_object GFileInfo *file_info = NULL;
 
-  root = (OstreeRepoFile*)ostree_repo_file_new_root (repo, resolved_commit);
-  if (!ostree_repo_file_ensure_resolved (root, error))
+  if (!ostree_repo_read_commit (repo, resolved_commit, &root, NULL, cancellable, error))
     goto out;
-  
+
   if (subpath)
-    subtree = (OstreeRepoFile*)g_file_resolve_relative_path ((GFile*)root, subpath);
+    subtree = g_file_resolve_relative_path (root, subpath);
   else
     subtree = g_object_ref (root);
 
-  file_info = g_file_query_info ((GFile*)subtree, OSTREE_GIO_FAST_QUERYINFO,
+  file_info = g_file_query_info (subtree, OSTREE_GIO_FAST_QUERYINFO,
                                  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                  cancellable, &tmp_error);
   if (!file_info)
@@ -89,7 +88,7 @@ process_one_checkout (OstreeRepo           *repo,
 
   if (!ostree_repo_checkout_tree (repo, opt_user_mode ? OSTREE_REPO_CHECKOUT_MODE_USER : 0,
                                   opt_union ? OSTREE_REPO_CHECKOUT_OVERWRITE_UNION_FILES : 0,
-                                  target, subtree, file_info, cancellable, error))
+                                  target, OSTREE_REPO_FILE (subtree), file_info, cancellable, error))
     goto out;
                       
   ret = TRUE;
@@ -177,13 +176,9 @@ ostree_builtin_checkout (int argc, char **argv, OstreeRepo *repo, GCancellable *
   gboolean ret = FALSE;
   const char *commit;
   const char *destination;
-  gs_free char *existing_commit = NULL;
   gs_free char *resolved_commit = NULL;
-  gs_free char *tmp_destination = NULL;
-  gs_unref_object GFileInfo *symlink_file_info = NULL;
   gs_unref_object GFile *checkout_target = NULL;
   gs_unref_object GFile *checkout_target_tmp = NULL;
-  gs_unref_object GFile *symlink_target = NULL;
 
   context = g_option_context_new ("COMMIT DESTINATION - Check out a commit into a filesystem tree");
   g_option_context_add_main_entries (context, options, NULL);
