@@ -61,34 +61,43 @@ import_one_object (OtLocalCloneData *data,
                    GError        **error)
 {
   gboolean ret = FALSE;
-  gs_unref_object GFile *content_path = NULL;
-  gs_unref_object GFileInfo *archive_info = NULL;
+  guint64 length;
+  gs_unref_object GInputStream *object = NULL;
+  
+  if (!ostree_repo_load_object_stream (data->src_repo, objtype, checksum,
+                                       &object, &length,
+                                       cancellable, error))
+    goto out;
 
   if (objtype == OSTREE_OBJECT_TYPE_FILE)
     {
-      guint64 length;
-      gs_unref_object GInputStream *file_object = NULL;
-
-      if (!ostree_repo_load_object_stream (data->src_repo, objtype, checksum,
-                                           &file_object, &length,
-                                           cancellable, error))
-        goto out;
-
-      if (!ostree_repo_stage_content_trusted (data->dest_repo, checksum,
-                                              file_object, length,
+      if (!ostree_repo_write_content_trusted (data->dest_repo, checksum,
+                                              object, length,
                                               cancellable, error))
         goto out;
     }
   else
     {
-      gs_unref_variant GVariant *metadata = NULL;
+      if (objtype == OSTREE_OBJECT_TYPE_COMMIT)
+        {
+          gs_unref_variant GVariant *detached_meta = NULL;
+          
+          if (!ostree_repo_read_commit_detached_metadata (data->src_repo,
+                                                          checksum, &detached_meta,
+                                                          cancellable, error))
+            goto out;
 
-      if (!ostree_repo_load_variant (data->src_repo, objtype, checksum, &metadata,
-                                     error))
-        goto out;
-
-      if (!ostree_repo_stage_metadata_trusted (data->dest_repo, objtype, checksum, metadata,
-                                               cancellable, error))
+          if (detached_meta)
+            {
+              if (!ostree_repo_write_commit_detached_metadata (data->src_repo,
+                                                               checksum, detached_meta,
+                                                               cancellable, error))
+                goto out;
+            }
+        }
+      if (!ostree_repo_write_metadata_stream_trusted (data->dest_repo, objtype,
+                                                      checksum, object, length,
+                                                      cancellable, error))
         goto out;
     }
 
@@ -163,16 +172,12 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
   GHashTableIter hash_iter;
   gpointer key, value;
   gboolean transaction_resuming = FALSE;
-  gs_unref_hashtable GHashTable *objects = NULL;
   gs_unref_object GFile *src_f = NULL;
   gs_unref_object GFile *src_repo_dir = NULL;
   gs_unref_object GFile *dest_repo_dir = NULL;
-  gs_unref_object GFile *src_dir = NULL;
-  gs_unref_object GFile *dest_dir = NULL;
   gs_unref_hashtable GHashTable *refs_to_clone = NULL;
   gs_unref_hashtable GHashTable *commits_to_clone = NULL;
   gs_unref_hashtable GHashTable *source_objects = NULL;
-  gs_unref_hashtable GHashTable *objects_to_copy = NULL;
   OtLocalCloneData datav = { 0, };
   OtLocalCloneData *data = &datav;
 
@@ -198,7 +203,7 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
   src_f = g_file_new_for_path (src_repo_path);
 
   data->src_repo = ostree_repo_new (src_f);
-  if (!ostree_repo_check (data->src_repo, error))
+  if (!ostree_repo_open (data->src_repo, cancellable, error))
     goto out;
 
   data->threadpool = ot_thread_pool_new_nproc (import_one_object_thread, data);
@@ -236,7 +241,7 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
         }
     }
 
-  if (!ostree_repo_prepare_transaction (data->dest_repo, FALSE, &transaction_resuming,
+  if (!ostree_repo_prepare_transaction (data->dest_repo, &transaction_resuming,
                                         cancellable, error))
     goto out;
 
@@ -251,8 +256,8 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
         {
           const char *checksum = value;
           
-          if (!ostree_repo_traverse_commit (data->src_repo, checksum, 0, source_objects,
-                                            cancellable, error))
+          if (!ostree_repo_traverse_commit_union (data->src_repo, checksum, 0, source_objects,
+                                                  cancellable, error))
             goto out;
         }
     }
@@ -263,9 +268,10 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
       while (g_hash_table_iter_next (&hash_iter, &key, &value))
         {
           const char *checksum = key;
+          gs_unref_hashtable GHashTable *tmp_source_objects = NULL;
 
-          if (!ostree_repo_traverse_commit (data->src_repo, checksum, 0, source_objects,
-                                            cancellable, error))
+          if (!ostree_repo_traverse_commit_union (data->src_repo, checksum, 0, source_objects,
+                                                  cancellable, error))
             goto out;
         }
     }
@@ -296,9 +302,6 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
         gs_console_end_status_line (data->console, NULL, NULL);
     }
 
-  if (!ostree_repo_commit_transaction (data->dest_repo, NULL, error))
-    goto out;
-
   g_print ("Writing %u refs\n", g_hash_table_size (refs_to_clone));
 
   g_hash_table_iter_init (&hash_iter, refs_to_clone);
@@ -307,11 +310,12 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
       const char *name = key;
       const char *checksum = value;
         
-      if (!ostree_repo_write_ref (data->dest_repo, opt_remote, name, checksum,
-                                  error))
-        goto out;
+      ostree_repo_transaction_set_ref (data->dest_repo, opt_remote, name, checksum);
     }
 
+  if (!ostree_repo_commit_transaction (data->dest_repo, NULL, NULL, error))
+    goto out;
+
   ret = TRUE;
  out:
   g_clear_pointer (&data->threadpool, (GDestroyNotify) g_thread_pool_free);
@@ -321,5 +325,6 @@ ostree_builtin_pull_local (int argc, char **argv, OstreeRepo *repo, GCancellable
     g_object_unref (data->dest_repo);
   if (context)
     g_option_context_free (context);
+  ostree_repo_abort_transaction (repo, cancellable, NULL);
   return ret;
 }
